@@ -88,6 +88,7 @@ def save_model(
     input_example: ModelInputExample = None,
     pip_requirements=None,
     extra_pip_requirements=None,
+    metadata=None,
     **kwargs,
 ):
     """
@@ -121,6 +122,10 @@ def save_model(
                           base64-encoded.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
 
     :param kwargs: kwargs to pass to ``Learner.save`` method.
 
@@ -163,6 +168,8 @@ def save_model(
         mlflow_model.signature = signature
     if input_example is not None:
         _save_example(mlflow_model, input_example, path)
+    if metadata is not None:
+        mlflow_model.metadata = metadata
 
     # ParamScheduler currently is not pickable
     # hence it is been removed before export and added again after export
@@ -176,7 +183,8 @@ def save_model(
         loader_module="mlflow.fastai",
         data=model_data_subpath,
         code=code_dir_subpath,
-        env=_CONDA_ENV_FILE_NAME,
+        conda_env=_CONDA_ENV_FILE_NAME,
+        python_env=_PYTHON_ENV_FILE_NAME,
     )
     mlflow_model.add_flavor(
         FLAVOR_NAME,
@@ -232,6 +240,7 @@ def log_model(
     await_registration_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
     pip_requirements=None,
     extra_pip_requirements=None,
+    metadata=None,
     **kwargs,
 ):
     """
@@ -273,6 +282,10 @@ def log_model(
                             waits for five minutes. Specify 0 or None to skip waiting.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     :return: A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
              metadata of the logged model.
 
@@ -322,6 +335,7 @@ def log_model(
         await_registration_for=await_registration_for,
         pip_requirements=pip_requirements,
         extra_pip_requirements=extra_pip_requirements,
+        metadata=metadata,
         **kwargs,
     )
 
@@ -541,7 +555,7 @@ def autolog(
         xb = learner.dls.train.one_batch()[: learner.dls.train.n_inp]
         infos = layer_info(learner, *xb)
         bs = find_bs(xb)
-        inp_sz = _print_shapes(map(lambda x: x.shape, xb), bs)
+        inp_sz = _print_shapes((x.shape for x in xb), bs)
         mlflow.log_param("input_size", inp_sz)
         mlflow.log_param("num_layers", len(infos))
 
