@@ -69,6 +69,7 @@ from mlflow.utils.autologging_utils import (
     MlflowAutologgingQueueingClient,
 )
 from mlflow.tracking._model_registry import DEFAULT_AWAIT_MAX_SLEEP_SECONDS
+from mlflow.sklearn import _SklearnTrainingSession
 
 FLAVOR_NAME = "lightgbm"
 
@@ -106,6 +107,7 @@ def save_model(
     input_example: ModelInputExample = None,
     pip_requirements=None,
     extra_pip_requirements=None,
+    metadata=None,
 ):
     """
     Save a LightGBM model to a path on the local file system.
@@ -139,6 +141,10 @@ def save_model(
                           base64-encoded.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     """
     import lightgbm as lgb
 
@@ -156,6 +162,8 @@ def save_model(
         mlflow_model.signature = signature
     if input_example is not None:
         _save_example(mlflow_model, input_example, path)
+    if metadata is not None:
+        mlflow_model.metadata = metadata
 
     # Save a LightGBM model
     _save_model(lgb_model, model_data_path)
@@ -165,7 +173,8 @@ def save_model(
         mlflow_model,
         loader_module="mlflow.lightgbm",
         data=model_data_subpath,
-        env=_CONDA_ENV_FILE_NAME,
+        conda_env=_CONDA_ENV_FILE_NAME,
+        python_env=_PYTHON_ENV_FILE_NAME,
         code=code_dir_subpath,
     )
     mlflow_model.add_flavor(
@@ -241,6 +250,7 @@ def log_model(
     await_registration_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
     pip_requirements=None,
     extra_pip_requirements=None,
+    metadata=None,
     **kwargs,
 ):
     """
@@ -280,6 +290,10 @@ def log_model(
                             waits for five minutes. Specify 0 or None to skip waiting.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     :param kwargs: kwargs to pass to `lightgbm.Booster.save_model`_ method.
     :return: A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
              metadata of the logged model.
@@ -296,6 +310,7 @@ def log_model(
         await_registration_for=await_registration_for,
         pip_requirements=pip_requirements,
         extra_pip_requirements=extra_pip_requirements,
+        metadata=metadata,
         **kwargs,
     )
 
@@ -477,11 +492,11 @@ def autolog(
             except Exception as e:
                 input_example_info = InputExampleInfo(error_msg=str(e))
 
-            setattr(self, "input_example_info", input_example_info)
+            self.input_example_info = input_example_info
 
         original(self, *args, **kwargs)
 
-    def train(_log_models, original, *args, **kwargs):
+    def train_impl(_log_models, original, *args, **kwargs):
         def record_eval_results(eval_results, metrics_logger):
             """
             Create a callback function that records evaluation results.
@@ -514,13 +529,13 @@ def autolog(
             ax.set_yticks(yloc)
             ax.set_yticklabels(features)
             ax.set_xlabel("Importance")
-            ax.set_title("Feature Importance ({})".format(importance_type))
+            ax.set_title(f"Feature Importance ({importance_type})")
             fig.tight_layout()
 
             tmpdir = tempfile.mkdtemp()
             try:
                 # pylint: disable=undefined-loop-variable
-                filepath = os.path.join(tmpdir, "feature_importance_{}.png".format(imp_type))
+                filepath = os.path.join(tmpdir, f"feature_importance_{imp_type}.png")
                 fig.savefig(filepath)
                 mlflow.log_artifact(filepath)
             finally:
@@ -617,7 +632,7 @@ def autolog(
             imp = dict(zip(features, importance.tolist()))
             tmpdir = tempfile.mkdtemp()
             try:
-                filepath = os.path.join(tmpdir, "feature_importance_{}.json".format(imp_type))
+                filepath = os.path.join(tmpdir, f"feature_importance_{imp_type}.json")
                 with open(filepath, "w") as f:
                     json.dump(imp, f, indent=2)
                 mlflow.log_artifact(filepath)
@@ -668,6 +683,13 @@ def autolog(
 
         return model
 
+    def train(_log_models, original, *args, **kwargs):
+        with _SklearnTrainingSession(estimator=lightgbm.train, allow_children=False) as t:
+            if t.should_log():
+                return train_impl(_log_models, original, *args, **kwargs)
+            else:
+                return original(*args, **kwargs)
+
     safe_patch(FLAVOR_NAME, lightgbm.Dataset, "__init__", __init__)
     safe_patch(
         FLAVOR_NAME, lightgbm, "train", functools.partial(train, log_models), manage_run=True
