@@ -10,6 +10,7 @@ NOTE:
 """
 import logging
 import os
+import pathlib
 import sys
 import traceback
 
@@ -20,6 +21,7 @@ from mlflow.exceptions import MlflowException
 from mlflow.models.signature import ModelSignature
 from mlflow.models.utils import ModelInputExample, _save_example
 from mlflow.utils import reraise
+from mlflow.utils.file_utils import path_to_local_file_uri
 from mlflow.utils.annotations import keyword_only
 
 FLAVOR_NAME = "mleap"
@@ -35,6 +37,7 @@ def log_model(
     registered_model_name=None,
     signature: ModelSignature = None,
     input_example: ModelInputExample = None,
+    metadata=None,
 ):
     """
     Log a Spark MLLib model in MLeap format as an MLflow artifact
@@ -73,6 +76,10 @@ def log_model(
                           model. The given example will be converted to a Pandas DataFrame and then
                           serialized to json using the Pandas split-oriented format. Bytes are
                           base64-encoded.
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
 
     :return: A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
              metadata of the logged model.
@@ -119,6 +126,7 @@ def log_model(
         registered_model_name=registered_model_name,
         signature=signature,
         input_example=input_example,
+        metadata=metadata,
     )
 
 
@@ -130,6 +138,7 @@ def save_model(
     mlflow_model=None,
     signature: ModelSignature = None,
     input_example: ModelInputExample = None,
+    metadata=None,
 ):
     """
     Save a Spark MLlib PipelineModel in MLeap format at a local path.
@@ -184,6 +193,10 @@ def save_model(
                           model. The given example will be converted to a Pandas DataFrame and then
                           serialized to json using the Pandas split-oriented format. Bytes are
                           base64-encoded.
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     """
     if mlflow_model is None:
         mlflow_model = Model()
@@ -195,6 +208,8 @@ def save_model(
         mlflow_model.signature = signature
     if input_example is not None:
         _save_example(mlflow_model, input_example, path)
+    if metadata is not None:
+        mlflow_model.metadata = metadata
     mlflow_model.save(os.path.join(path, MLMODEL_FILE_NAME))
 
 
@@ -237,13 +252,19 @@ def add_to_model(mlflow_model, path, spark_model, sample_input):
     mleap_datapath_sub = os.path.join("mleap", "model")
     mleap_datapath_full = os.path.join(path, mleap_datapath_sub)
     if os.path.exists(mleap_path_full):
-        raise Exception(
-            "MLeap model data path already exists at: {path}".format(path=mleap_path_full)
-        )
+        raise Exception(f"MLeap model data path already exists at: {mleap_path_full}")
     os.makedirs(mleap_path_full)
 
     dataset = spark_model.transform(sample_input)
-    model_path = "file:{mp}".format(mp=mleap_datapath_full)
+    if os.name == "nt":
+        # NB: On Windows, MLeap requires the "file://" prefix in order to correctly
+        # parse the model data path, even though the result is not a correct URI.
+        # None of "file:", "file:/", or "file:///", which would be canonically correct,
+        # work properly
+        model_path = "file://" + str(pathlib.Path(mleap_datapath_full).as_posix())
+    else:
+        model_path = path_to_local_file_uri(mleap_datapath_full)
+
     try:
         spark_model.serializeToBundle(path=model_path, dataset=dataset)
     except Py4JError:
