@@ -10,6 +10,7 @@ import uuid
 from typing import Any, Dict, Optional, Union, Callable
 
 import mlflow
+from mlflow.artifacts import download_artifacts
 from mlflow.exceptions import MlflowException
 from mlflow.tracking._model_registry import DEFAULT_AWAIT_MAX_SLEEP_SECONDS
 from mlflow.utils.file_utils import TempDir
@@ -17,7 +18,7 @@ from mlflow.utils.databricks_utils import get_databricks_runtime
 
 _logger = logging.getLogger(__name__)
 
-
+# NOTE: The MLMODEL_FILE_NAME constant is considered @developer_stable
 MLMODEL_FILE_NAME = "MLmodel"
 _LOG_MODEL_METADATA_WARNING_TEMPLATE = (
     "Logging model metadata to the tracking server has failed, possibly due older "
@@ -28,6 +29,7 @@ _LOG_MODEL_METADATA_WARNING_TEMPLATE = (
     "1.7.0 or above. Set logging level to DEBUG via "
     '`logging.getLogger("mlflow").setLevel(logging.DEBUG)` to see the full traceback.'
 )
+# NOTE: The _MLFLOW_VERSION_KEY constant is considered @developer_stable
 _MLFLOW_VERSION_KEY = "mlflow_version"
 
 
@@ -48,6 +50,7 @@ class ModelInfo:
         utc_time_created: str,
         mlflow_version: str,
         signature_dict: Optional[Dict[str, Any]] = None,
+        metadata: Optional[Dict[str, Any]] = None,
     ):
         self._artifact_path = artifact_path
         self._flavors = flavors
@@ -59,6 +62,7 @@ class ModelInfo:
         self._signature = signature
         self._utc_time_created = utc_time_created
         self._mlflow_version = mlflow_version
+        self._metadata = metadata
 
     @property
     def artifact_path(self):
@@ -157,6 +161,44 @@ class ModelInfo:
         """
         return self._mlflow_version
 
+    @property
+    def metadata(self) -> Optional[Dict[str, Any]]:
+        """
+        .. Note::
+            Experimental: This method may change or be removed in a future release without warning.
+
+        User defined metadata added to the model.
+
+        .. code-block:: python
+            :caption: Example
+
+            # Create and log a model with metadata to the Model Registry
+
+            from sklearn import datasets
+            from sklearn.ensemble import RandomForestClassifier
+            import mlflow
+
+            with mlflow.start_run():
+                iris = datasets.load_iris()
+                clf = RandomForestClassifier()
+                clf.fit(iris.data, iris.target)
+                mlflow.sklearn.log_model(clf, "iris_rf",
+                                         registered_model_name="model-with-metadata",
+                                         metadata={"metadata_key": "metadata_value"})
+
+            # model uri for the above model
+            model_uri = "models:/model-with-metadata/1"
+
+            # Load the model and access the custom metadata from its ModelInfo object
+            model = mlflow.pyfunc.load_model(model_uri=model_uri)
+            assert model.metadata.get_model_info().metadata["metadata_key"] == "metadata_value"
+
+            # Load the ModelInfo and access the custom metadata
+            model_info = mlflow.models.get_model_info(model_uri=model_uri)
+            assert model_info.metadata["metadata_key"] == "metadata_value"
+        """
+        return self._metadata
+
 
 class Model:
     """
@@ -174,6 +216,7 @@ class Model:
         saved_input_example_info: Dict[str, Any] = None,
         model_uuid: Union[str, Callable, None] = lambda: uuid.uuid4().hex,
         mlflow_version: Union[str, None] = mlflow.version.VERSION,
+        metadata: Optional[Dict[str, Any]] = None,
         **kwargs,
     ):
         # store model id instead of run_id and path to avoid confusion when model gets exported
@@ -187,6 +230,7 @@ class Model:
         self.saved_input_example_info = saved_input_example_info
         self.model_uuid = model_uuid() if callable(model_uuid) else model_uuid
         self.mlflow_version = mlflow_version
+        self.metadata = metadata
         self.__dict__.update(kwargs)
 
     def __eq__(self, other):
@@ -223,6 +267,45 @@ class Model:
         self.flavors[name] = params
         return self
 
+    @property
+    def metadata(self) -> Optional[Dict[str, Any]]:
+        """
+        .. Note::
+            Experimental: This method may change or be removed in a future release without warning.
+
+        Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+        .. code-block:: python
+            :caption: Example
+
+            # Create and log a model with metadata to the Model Registry
+
+            from sklearn import datasets
+            from sklearn.ensemble import RandomForestClassifier
+            import mlflow
+
+            with mlflow.start_run():
+                iris = datasets.load_iris()
+                clf = RandomForestClassifier()
+                clf.fit(iris.data, iris.target)
+                mlflow.sklearn.log_model(clf, "iris_rf",
+                                         registered_model_name="model-with-metadata",
+                                         metadata={"metadata_key": "metadata_value"})
+
+            # model uri for the above model
+            model_uri = "models:/model-with-metadata/1"
+
+            # Load the model and access the custom metadata
+            model = mlflow.pyfunc.load_model(model_uri=model_uri)
+            assert model.metadata.metadata["metadata_key"] == "metadata_value"
+        """
+        return self._metadata
+
+    @metadata.setter
+    def metadata(self, value: Optional[Dict[str, Any]]):
+        # pylint: disable=attribute-defined-outside-init
+        self._metadata = value
+
     @property
     def signature(self):  # -> Optional[ModelSignature]
         return self._signature
@@ -261,6 +344,7 @@ class Model:
             signature=self.signature,
             utc_time_created=self.utc_time_created,
             mlflow_version=self.mlflow_version,
+            metadata=self.metadata,
         )
 
     def to_dict(self):
@@ -275,6 +359,8 @@ class Model:
             res["saved_input_example_info"] = self.saved_input_example_info
         if self.mlflow_version is None and _MLFLOW_VERSION_KEY in res:
             res.pop(_MLFLOW_VERSION_KEY)
+        if self.metadata is not None:
+            res["metadata"] = self.metadata
         return res
 
     def to_yaml(self, stream=None):
@@ -295,7 +381,27 @@ class Model:
 
     @classmethod
     def load(cls, path):
-        """Load a model from its YAML representation."""
+        """
+        Load a model from its YAML representation.
+
+        :param path: A local filesystem path or URI referring to the MLmodel YAML file
+                     representation of the :py:class:`Model` object or to the directory containing
+                     the MLmodel YAML file representation.
+        :return: An instance of :py:class:`Model`.
+
+
+        .. code-block:: python
+            :caption: example
+
+            from mlflow.models import Model
+
+            # Load the Model object from a local MLmodel file
+            model1 = Model.load("~/path/to/my/MLmodel")
+
+            # Load the Model object from a remote model directory
+            model2 = Model.load("s3://mybucket/path/to/my/model")
+        """
+        path = download_artifacts(artifact_uri=path)
         if os.path.isdir(path):
             path = os.path.join(path, MLMODEL_FILE_NAME)
         with open(path) as f:
@@ -326,6 +432,7 @@ class Model:
         flavor,
         registered_model_name=None,
         await_registration_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
+        metadata=None,
         **kwargs,
     ):
         """
@@ -361,6 +468,12 @@ class Model:
                             being created and is in ``READY`` status. By default, the function
                             waits for five minutes. Specify 0 or None to skip waiting.
 
+        :param metadata: Custom metadata dictionary passed to the model and stored in
+                         the MLmodel file.
+
+                         .. Note:: Experimental: This parameter may change or be removed in a
+                                                 future release without warning.
+
         :param kwargs: Extra args passed to the model flavor.
 
         :return: A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
@@ -369,7 +482,7 @@ class Model:
         with TempDir() as tmp:
             local_path = tmp.path("model")
             run_id = mlflow.tracking.fluent._get_or_start_run().info.run_id
-            mlflow_model = cls(artifact_path=artifact_path, run_id=run_id)
+            mlflow_model = cls(artifact_path=artifact_path, run_id=run_id, metadata=metadata)
             flavor.save_model(path=local_path, mlflow_model=mlflow_model, **kwargs)
             mlflow.tracking.fluent.log_artifacts(local_path, mlflow_model.artifact_path)
             try:
@@ -382,7 +495,7 @@ class Model:
             if registered_model_name is not None:
                 run_id = mlflow.tracking.fluent.active_run().info.run_id
                 mlflow.register_model(
-                    "runs:/%s/%s" % (run_id, mlflow_model.artifact_path),
+                    "runs:/{}/{}".format(run_id, mlflow_model.artifact_path),
                     registered_model_name,
                     await_registration_for=await_registration_for,
                 )
@@ -447,4 +560,5 @@ def get_model_info(model_uri: str) -> ModelInfo:
         signature=model_meta.signature,
         utc_time_created=model_meta.utc_time_created,
         mlflow_version=model_meta.mlflow_version,
+        metadata=model_meta.metadata,
     )
