@@ -6,13 +6,17 @@ from typing import Union, Any, Dict, List
 import numpy as np
 import pandas as pd
 
-from mlflow.exceptions import MlflowException
+from mlflow.exceptions import MlflowException, INVALID_PARAMETER_VALUE
 from mlflow.models import Model
 from mlflow.store.artifact.utils.models import get_model_name_and_version
-from mlflow.utils.annotations import experimental
 from mlflow.types import DataType, Schema, TensorSpec
 from mlflow.types.utils import TensorsNotSupportedException, clean_tensor_type
-from mlflow.utils.proto_json_utils import NumpyEncoder, _dataframe_from_json, parse_tf_serving_input
+from mlflow.utils.annotations import experimental
+from mlflow.utils.proto_json_utils import (
+    NumpyEncoder,
+    dataframe_from_raw_json,
+    parse_tf_serving_input,
+)
 from mlflow.utils.uri import get_databricks_profile_uri_from_artifact_uri
 
 try:
@@ -131,7 +135,7 @@ class _Example:
                 for i, x in enumerate(input_ex):
                     if isinstance(x, np.ndarray) and len(x.shape) > 1:
                         raise TensorsNotSupportedException(
-                            "Row '{0}' has shape {1}".format(i, x.shape)
+                            "Row '{}' has shape {}".format(i, x.shape)
                         )
                 if all(_is_scalar(x) for x in input_ex):
                     input_ex = pd.DataFrame([input_ex], columns=range(len(input_ex)))
@@ -229,9 +233,7 @@ def _read_example(mlflow_model: Model, path: str):
         return None
     example_type = mlflow_model.saved_input_example_info["type"]
     if example_type not in ["dataframe", "ndarray", "sparse_matrix_csc", "sparse_matrix_csr"]:
-        raise MlflowException(
-            "This version of mlflow can not load example of type {}".format(example_type)
-        )
+        raise MlflowException(f"This version of mlflow can not load example of type {example_type}")
     input_schema = mlflow_model.signature.inputs if mlflow_model.signature is not None else None
     path = os.path.join(path, mlflow_model.saved_input_example_info["artifact_path"])
     if example_type == "ndarray":
@@ -239,17 +241,17 @@ def _read_example(mlflow_model: Model, path: str):
     elif example_type in ["sparse_matrix_csc", "sparse_matrix_csr"]:
         return _read_sparse_matrix_from_json(path, example_type)
     else:
-        return _dataframe_from_json(path, schema=input_schema, precise_float=True)
+        return dataframe_from_raw_json(path, schema=input_schema)
 
 
 def _read_tensor_input_from_json(path, schema=None):
-    with open(path, "r") as handle:
+    with open(path) as handle:
         inp_dict = json.load(handle)
         return parse_tf_serving_input(inp_dict, schema)
 
 
 def _read_sparse_matrix_from_json(path, example_type):
-    with open(path, "r") as handle:
+    with open(path) as handle:
         matrix_data = json.load(handle)
         data = matrix_data["data"]
         indices = matrix_data["indices"]
@@ -289,30 +291,41 @@ def _enforce_tensor_spec(
     Enforce the input tensor shape and type matches the provided tensor spec.
     """
     expected_shape = tensor_spec.shape
+    expected_type = tensor_spec.type
     actual_shape = values.shape
-
     actual_type = values.dtype if isinstance(values, np.ndarray) else values.data.dtype
 
+    # This logic is for handling "ragged" arrays. The first check is for a standard numpy shape
+    # representation of a ragged array. The second is for handling a more manual specification
+    # of shape while support an input which is a ragged array.
+    if len(expected_shape) == 1 and expected_shape[0] == -1 and expected_type == np.dtype("O"):
+        # Sample spec: Tensor('object', (-1,))
+        # Will pass on any provided input
+        return values
+    if (
+        len(expected_shape) > 1
+        and -1 in expected_shape[1:]
+        and len(actual_shape) == 1
+        and actual_type == np.dtype("O")
+    ):
+        # Sample spec: Tensor('float64', (-1, -1, -1, 3))
+        # Will pass on inputs which are ragged arrays: shape==(x,), dtype=='object'
+        return values
+
     if len(expected_shape) != len(actual_shape):
         raise MlflowException(
-            "Shape of input {0} does not match expected shape {1}.".format(
-                actual_shape, expected_shape
-            )
+            f"Shape of input {actual_shape} does not match expected shape {expected_shape}."
         )
     for expected, actual in zip(expected_shape, actual_shape):
         if expected == -1:
             continue
         if expected != actual:
             raise MlflowException(
-                "Shape of input {0} does not match expected shape {1}.".format(
-                    actual_shape, expected_shape
-                )
+                f"Shape of input {actual_shape} does not match expected shape {expected_shape}."
             )
-    if clean_tensor_type(actual_type) != tensor_spec.type:
+    if clean_tensor_type(actual_type) != expected_type:
         raise MlflowException(
-            "dtype of input {0} does not match expected dtype {1}".format(
-                values.dtype, tensor_spec.type
-            )
+            f"dtype of input {actual_type} does not match expected dtype {expected_type}"
         )
     return values
 
@@ -369,10 +382,10 @@ def _enforce_mlflow_datatype(name, values: pd.Series, t: DataType):
         # DataFrame. To respect the original typing, we convert the column to datetime.
         try:
             return values.astype(np.datetime64, errors="raise")
-        except ValueError:
+        except ValueError as e:
             raise MlflowException(
-                "Failed to convert column {0} from type {1} to {2}.".format(name, values.dtype, t)
-            )
+                "Failed to convert column {} from type {} to {}.".format(name, values.dtype, t)
+            ) from e
     if t == DataType.double and values.dtype == decimal.Decimal:
         # NB: Pyspark Decimal column get converted to decimal.Decimal when converted to pandas
         # DataFrame. In order to support decimal data training from spark data frame, we add this
@@ -381,7 +394,7 @@ def _enforce_mlflow_datatype(name, values: pd.Series, t: DataType):
             return pd.to_numeric(values, errors="raise")
         except ValueError:
             raise MlflowException(
-                "Failed to convert column {0} from type {1} to {2}.".format(name, values.dtype, t)
+                "Failed to convert column {} from type {} to {}.".format(name, values.dtype, t)
             )
 
     numpy_type = t.to_numpy()
@@ -423,25 +436,90 @@ def _enforce_mlflow_datatype(name, values: pd.Series, t: DataType):
             )
 
         raise MlflowException(
-            "Incompatible input types for column {0}. "
-            "Can not safely convert {1} to {2}.{3}".format(name, values.dtype, numpy_type, hint)
+            "Incompatible input types for column {}. "
+            "Can not safely convert {} to {}.{}".format(name, values.dtype, numpy_type, hint)
         )
 
 
-def _enforce_col_schema(pfInput: PyFuncInput, input_schema: Schema):
+def _enforce_col_schema(pf_input: PyFuncInput, input_schema: Schema):
     """Enforce the input columns conform to the model's column-based signature."""
     if input_schema.has_input_names():
         input_names = input_schema.input_names()
     else:
-        input_names = pfInput.columns[: len(input_schema.inputs)]
+        input_names = pf_input.columns[: len(input_schema.inputs)]
     input_types = input_schema.input_types()
-    new_pfInput = pd.DataFrame()
+    new_pf_input = pd.DataFrame()
     for i, x in enumerate(input_names):
-        new_pfInput[x] = _enforce_mlflow_datatype(x, pfInput[x], input_types[i])
-    return new_pfInput
+        new_pf_input[x] = _enforce_mlflow_datatype(x, pf_input[x], input_types[i])
+    return new_pf_input
 
 
-def _enforce_tensor_schema(pfInput: PyFuncInput, input_schema: Schema):
+def _reshape_and_cast_pandas_column_values(name, pd_series, tensor_spec):
+    if tensor_spec.shape[0] != -1 or -1 in tensor_spec.shape[1:]:
+        raise MlflowException(
+            "For pandas dataframe input, the first dimension of shape must be a variable "
+            "dimension and other dimensions must be fixed, but in model signature the shape "
+            f"of {'input ' + name if name else 'the unnamed input'} is {tensor_spec.shape}."
+        )
+
+    if np.isscalar(pd_series[0]):
+        if tensor_spec.shape != (-1,):
+            raise MlflowException(
+                f"The input pandas dataframe column '{name}' contains scalar "
+                "values, which requires the shape to be (-1,), but got tensor spec "
+                f"shape of {tensor_spec.shape}.",
+                error_code=INVALID_PARAMETER_VALUE,
+            )
+        return _enforce_tensor_spec(np.array(pd_series, dtype=tensor_spec.type), tensor_spec)
+    elif isinstance(pd_series[0], list) and np.isscalar(pd_series[0][0]):
+        # If the pandas column contains list type values,
+        # in this case, the shape and type information is lost,
+        # so do not enforce the shape and type, instead,
+        # reshape the array value list to the required shape, and cast value type to
+        # required type.
+        reshape_err_msg = (
+            f"The value in the Input DataFrame column '{name}' could not be converted to the "
+            f"expected shape of: '{tensor_spec.shape}'. Ensure that each of the input list "
+            "elements are of uniform length and that the data can be coerced to the tensor "
+            f"type '{tensor_spec.type}'"
+        )
+        try:
+            flattened_numpy_arr = np.vstack(pd_series.tolist())
+            reshaped_numpy_arr = flattened_numpy_arr.reshape(tensor_spec.shape).astype(
+                tensor_spec.type
+            )
+        except ValueError:
+            raise MlflowException(reshape_err_msg, error_code=INVALID_PARAMETER_VALUE)
+        if len(reshaped_numpy_arr) != len(pd_series):
+            raise MlflowException(reshape_err_msg, error_code=INVALID_PARAMETER_VALUE)
+        return reshaped_numpy_arr
+    elif isinstance(pd_series[0], np.ndarray):
+        reshape_err_msg = (
+            f"The value in the Input DataFrame column '{name}' could not be converted to the "
+            f"expected shape of: '{tensor_spec.shape}'. Ensure that each of the input numpy "
+            "array elements are of uniform length and can be reshaped to above expected shape."
+        )
+        try:
+            # Because numpy array includes precise type information, so we don't convert type
+            # here, so that in following schema validation we can have strict type check on
+            # numpy array column.
+            reshaped_numpy_arr = np.vstack(pd_series.tolist()).reshape(tensor_spec.shape)
+        except ValueError:
+            raise MlflowException(reshape_err_msg, error_code=INVALID_PARAMETER_VALUE)
+        if len(reshaped_numpy_arr) != len(pd_series):
+            raise MlflowException(reshape_err_msg, error_code=INVALID_PARAMETER_VALUE)
+        return reshaped_numpy_arr
+    else:
+        raise MlflowException(
+            "Because the model signature requires tensor spec input, the input "
+            "pandas dataframe values should be either scalar value, python list "
+            "containing scalar values or numpy array containing scalar values, "
+            "other types are not supported.",
+            error_code=INVALID_PARAMETER_VALUE,
+        )
+
+
+def _enforce_tensor_schema(pf_input: PyFuncInput, input_schema: Schema):
     """Enforce the input tensor(s) conforms to the model's tensor-based signature."""
 
     def _is_sparse_matrix(x):
@@ -451,45 +529,71 @@ def _enforce_tensor_schema(pfInput: PyFuncInput, input_schema: Schema):
         return isinstance(x, (csr_matrix, csc_matrix))
 
     if input_schema.has_input_names():
-        if isinstance(pfInput, dict):
-            new_pfInput = dict()
+        if isinstance(pf_input, dict):
+            new_pf_input = {}
             for col_name, tensor_spec in zip(input_schema.input_names(), input_schema.inputs):
-                if not isinstance(pfInput[col_name], np.ndarray):
+                if not isinstance(pf_input[col_name], np.ndarray):
                     raise MlflowException(
                         "This model contains a tensor-based model signature with input names,"
                         " which suggests a dictionary input mapping input name to a numpy"
-                        " array, but a dict with value type {0} was found.".format(
-                            type(pfInput[col_name])
-                        )
+                        " array, but a dict with value type {} was found.".format(
+                            type(pf_input[col_name])
+                        ),
+                        error_code=INVALID_PARAMETER_VALUE,
                     )
-                new_pfInput[col_name] = _enforce_tensor_spec(pfInput[col_name], tensor_spec)
-        elif isinstance(pfInput, pd.DataFrame):
-            new_pfInput = dict()
+                new_pf_input[col_name] = _enforce_tensor_spec(pf_input[col_name], tensor_spec)
+        elif isinstance(pf_input, pd.DataFrame):
+            new_pf_input = {}
             for col_name, tensor_spec in zip(input_schema.input_names(), input_schema.inputs):
-                new_pfInput[col_name] = _enforce_tensor_spec(
-                    np.array(pfInput[col_name], dtype=tensor_spec.type), tensor_spec
+                pd_series = pf_input[col_name]
+                new_pf_input[col_name] = _reshape_and_cast_pandas_column_values(
+                    col_name, pd_series, tensor_spec
                 )
         else:
             raise MlflowException(
                 "This model contains a tensor-based model signature with input names, which"
-                " suggests a dictionary input mapping input name to tensor, but an input of"
-                " type {0} was found.".format(type(pfInput))
+                " suggests a dictionary input mapping input name to tensor, or a pandas"
+                " DataFrame input containing columns mapping input name to flattened list value"
+                f" from tensor, but an input of type {type(pf_input)} was found.",
+                error_code=INVALID_PARAMETER_VALUE,
             )
     else:
-        if isinstance(pfInput, pd.DataFrame):
-            new_pfInput = _enforce_tensor_spec(pfInput.to_numpy(), input_schema.inputs[0])
-        elif isinstance(pfInput, np.ndarray) or _is_sparse_matrix(pfInput):
-            new_pfInput = _enforce_tensor_spec(pfInput, input_schema.inputs[0])
+        tensor_spec = input_schema.inputs[0]
+        if isinstance(pf_input, pd.DataFrame):
+            num_input_columns = len(pf_input.columns)
+            if pf_input.empty:
+                raise MlflowException("Input DataFrame is empty.")
+            elif num_input_columns == 1:
+                new_pf_input = _reshape_and_cast_pandas_column_values(
+                    None, pf_input[pf_input.columns[0]], tensor_spec
+                )
+            else:
+                if tensor_spec.shape != (-1, num_input_columns):
+                    raise MlflowException(
+                        "This model contains a model signature with an unnamed input. Since the "
+                        "input data is a pandas DataFrame containing multiple columns, "
+                        "the input shape must be of the structure "
+                        "(-1, number_of_dataframe_columns). "
+                        f"Instead, the input DataFrame passed had {num_input_columns} columns and "
+                        f"an input shape of {tensor_spec.shape} with all values within the "
+                        "DataFrame of scalar type. Please adjust the passed in DataFrame to "
+                        "match the expected structure",
+                        error_code=INVALID_PARAMETER_VALUE,
+                    )
+                new_pf_input = _enforce_tensor_spec(pf_input.to_numpy(), tensor_spec)
+        elif isinstance(pf_input, np.ndarray) or _is_sparse_matrix(pf_input):
+            new_pf_input = _enforce_tensor_spec(pf_input, tensor_spec)
         else:
             raise MlflowException(
                 "This model contains a tensor-based model signature with no input names,"
-                " which suggests a numpy array input, but an input of type {0} was"
-                " found.".format(type(pfInput))
+                " which suggests a numpy array input or a pandas dataframe input with"
+                f" proper column values, but an input of type {type(pf_input)} was found.",
+                error_code=INVALID_PARAMETER_VALUE,
             )
-    return new_pfInput
+    return new_pf_input
 
 
-def _enforce_schema(pfInput: PyFuncInput, input_schema: Schema):
+def _enforce_schema(pf_input: PyFuncInput, input_schema: Schema):
     """
     Enforces the provided input matches the model's input schema,
 
@@ -502,21 +606,21 @@ def _enforce_schema(pfInput: PyFuncInput, input_schema: Schema):
     For tensor-based signatures, we make sure the shape and type of the input matches the shape
     and type specified in model's input schema.
     """
-    if isinstance(pfInput, pd.Series):
-        pfInput = pd.DataFrame(pfInput)
+    if isinstance(pf_input, pd.Series):
+        pf_input = pd.DataFrame(pf_input)
     if not input_schema.is_tensor_spec():
-        if isinstance(pfInput, (list, np.ndarray, dict, pd.Series)):
+        if isinstance(pf_input, (list, np.ndarray, dict, pd.Series)):
             try:
-                pfInput = pd.DataFrame(pfInput)
+                pf_input = pd.DataFrame(pf_input)
             except Exception as e:
                 raise MlflowException(
                     "This model contains a column-based signature, which suggests a DataFrame"
                     " input. There was an error casting the input data to a DataFrame:"
-                    " {0}".format(str(e))
+                    " {}".format(str(e))
                 )
-        if not isinstance(pfInput, pd.DataFrame):
+        if not isinstance(pf_input, pd.DataFrame):
             raise MlflowException(
-                "Expected input to be DataFrame or list. Found: %s" % type(pfInput).__name__
+                "Expected input to be DataFrame or list. Found: %s" % type(pf_input).__name__
             )
 
     if input_schema.has_input_names():
@@ -524,14 +628,14 @@ def _enforce_schema(pfInput: PyFuncInput, input_schema: Schema):
         input_names = input_schema.input_names()
         expected_cols = set(input_names)
         actual_cols = set()
-        if len(expected_cols) == 1 and isinstance(pfInput, np.ndarray):
+        if len(expected_cols) == 1 and isinstance(pf_input, np.ndarray):
             # for schemas with a single column, match input with column
-            pfInput = {input_names[0]: pfInput}
+            pf_input = {input_names[0]: pf_input}
             actual_cols = expected_cols
-        elif isinstance(pfInput, pd.DataFrame):
-            actual_cols = set(pfInput.columns)
-        elif isinstance(pfInput, dict):
-            actual_cols = set(pfInput.keys())
+        elif isinstance(pf_input, pd.DataFrame):
+            actual_cols = set(pf_input.columns)
+        elif isinstance(pf_input, dict):
+            actual_cols = set(pf_input.keys())
         missing_cols = expected_cols - actual_cols
         extra_cols = actual_cols - expected_cols
         # Preserve order from the original columns, since missing/extra columns are likely to
@@ -539,25 +643,25 @@ def _enforce_schema(pfInput: PyFuncInput, input_schema: Schema):
         missing_cols = [c for c in input_names if c in missing_cols]
         extra_cols = [c for c in actual_cols if c in extra_cols]
         if missing_cols:
-            message = "Model is missing inputs {0}.".format(missing_cols)
+            message = f"Model is missing inputs {missing_cols}."
             if extra_cols:
-                message += " Note that there were extra inputs: {0}".format(extra_cols)
+                message += f" Note that there were extra inputs: {extra_cols}"
             raise MlflowException(message)
     elif not input_schema.is_tensor_spec():
         # The model signature does not specify column names => we can only verify column count.
-        num_actual_columns = len(pfInput.columns)
+        num_actual_columns = len(pf_input.columns)
         if num_actual_columns < len(input_schema.inputs):
             raise MlflowException(
                 "Model inference is missing inputs. The model signature declares "
-                "{0} inputs  but the provided value only has "
-                "{1} inputs. Note: the inputs were not named in the signature so we can "
+                "{} inputs  but the provided value only has "
+                "{} inputs. Note: the inputs were not named in the signature so we can "
                 "only verify their count.".format(len(input_schema.inputs), num_actual_columns)
             )
 
     return (
-        _enforce_tensor_schema(pfInput, input_schema)
+        _enforce_tensor_schema(pf_input, input_schema)
         if input_schema.is_tensor_spec()
-        else _enforce_col_schema(pfInput, input_schema)
+        else _enforce_col_schema(pf_input, input_schema)
     )
 
 
