@@ -22,7 +22,6 @@ from mlflow.exceptions import MlflowException
 from mlflow.models.signature import ModelSignature
 from mlflow.models.utils import ModelInputExample, _save_example
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
-from mlflow.utils.annotations import experimental
 from mlflow.utils.environment import (
     _mlflow_conda_env,
     _validate_env_arguments,
@@ -69,7 +68,6 @@ def get_default_pip_requirements():
     )
 
 
-@experimental
 def get_default_conda_env():
     """
     :return: The default Conda environment for MLflow Models produced by calls to
@@ -78,7 +76,6 @@ def get_default_conda_env():
     return _mlflow_conda_env(additional_pip_deps=get_default_pip_requirements())
 
 
-@experimental
 @format_docstring(LOG_MODEL_PARAM_DOCS.format(package_name=FLAVOR_NAME))
 def save_model(
     onnx_model,
@@ -91,6 +88,7 @@ def save_model(
     pip_requirements=None,
     extra_pip_requirements=None,
     onnx_execution_providers=None,
+    metadata=None,
 ):
     """
     Save an ONNX model to a path on the local file system.
@@ -130,6 +128,10 @@ def save_model(
                                      This uses GPU preferentially over CPU.
                                      See onnxruntime API for further descriptions:
                                      https://onnxruntime.ai/docs/execution-providers/
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     """
     import onnx
 
@@ -148,6 +150,8 @@ def save_model(
         mlflow_model.signature = signature
     if input_example is not None:
         _save_example(mlflow_model, input_example, path)
+    if metadata is not None:
+        mlflow_model.metadata = metadata
     model_data_subpath = "model.onnx"
     model_data_path = os.path.join(path, model_data_subpath)
 
@@ -158,7 +162,8 @@ def save_model(
         mlflow_model,
         loader_module="mlflow.onnx",
         data=model_data_subpath,
-        env=_CONDA_ENV_FILE_NAME,
+        conda_env=_CONDA_ENV_FILE_NAME,
+        python_env=_PYTHON_ENV_FILE_NAME,
         code=code_dir_subpath,
     )
     mlflow_model.add_flavor(
@@ -268,7 +273,6 @@ class _OnnxModelWrapper:
                     feeds[input_name] = feed.astype(np.float32)
         return feeds
 
-    @experimental
     def predict(self, data):
         """
         :param data: Either a pandas DataFrame, numpy.ndarray or a dictionary.
@@ -306,9 +310,9 @@ class _OnnxModelWrapper:
                     "input. "
                     "Numpy arrays can only be used as input for MLflow ONNX "
                     "models that have a single input. This model requires "
-                    "{0} inputs. Please pass in data as either a "
+                    "{} inputs. Please pass in data as either a "
                     "dictionary or a DataFrame with the following tensors"
-                    ": {1}.".format(len(self.inputs), inputs)
+                    ": {}.".format(len(self.inputs), inputs)
                 )
             feed_dict = {self.inputs[0][0]: data}
         elif isinstance(data, pd.DataFrame):
@@ -355,7 +359,6 @@ def _load_pyfunc(path):
     return _OnnxModelWrapper(path)
 
 
-@experimental
 def load_model(model_uri, dst_path=None):
     """
     Load an ONNX model from a local file or a run.
@@ -386,7 +389,6 @@ def load_model(model_uri, dst_path=None):
     return _load_model(model_file=onnx_model_artifacts_path)
 
 
-@experimental
 @format_docstring(LOG_MODEL_PARAM_DOCS.format(package_name=FLAVOR_NAME))
 def log_model(
     onnx_model,
@@ -400,6 +402,7 @@ def log_model(
     pip_requirements=None,
     extra_pip_requirements=None,
     onnx_execution_providers=None,
+    metadata=None,
 ):
     """
     Log an ONNX model as an MLflow artifact for the current run.
@@ -444,6 +447,10 @@ def log_model(
                                      This uses GPU preferentially over CPU.
                                      See onnxruntime API for further descriptions:
                                      https://onnxruntime.ai/docs/execution-providers/
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     :return: A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
              metadata of the logged model.
     """
@@ -460,4 +467,5 @@ def log_model(
         pip_requirements=pip_requirements,
         extra_pip_requirements=extra_pip_requirements,
         onnx_execution_providers=onnx_execution_providers,
+        metadata=metadata,
     )
