@@ -28,9 +28,9 @@ metadata (MLmodel file). You can score the model by calling the :py:func:`predic
     List[Any], Dict[str, Any]]
   ) -> [numpy.ndarray | pandas.(Series | DataFrame) | List]
 
-All PyFunc models will support `pandas.DataFrame` as input and DL PyFunc models will also support
-tensor inputs in the form of Dict[str, numpy.ndarray] (named tensors) and `numpy.ndarrays`
-(unnamed tensors).
+All PyFunc models will support `pandas.DataFrame` as input and PyFunc deep learning models will
+also support tensor inputs in the form of Dict[str, numpy.ndarray] (named tensors) and
+`numpy.ndarrays` (unnamed tensors).
 
 
 .. _pyfunc-filesystem-format:
@@ -177,8 +177,8 @@ Workflows
    The ``loader_module`` parameter specifies the name of your loader module.
 
    For an example loader module implementation, refer to the `loader module
-   implementation in mlflow.keras <https://github.com/mlflow/mlflow/blob/
-   74d75109aaf2975f5026104d6125bb30f4e3f744/mlflow/keras.py#L157-L187>`_.
+   implementation in mlflow.sklearn <https://github.com/mlflow/mlflow/blob/
+   74d75109aaf2975f5026104d6125bb30f4e3f744/mlflow/sklearn.py#L200-L205>`_.
 
 .. _pyfunc-create-custom-selecting-workflow:
 
@@ -225,8 +225,10 @@ import yaml
 
 import mlflow
 import mlflow.pyfunc.model
+from mlflow.environment_variables import MLFLOW_SCORING_SERVER_REQUEST_TIMEOUT
 from mlflow.exceptions import MlflowException
 from mlflow.models import Model, ModelSignature, ModelInputExample
+from mlflow.models.flavor_backend_registry import get_flavor_backend
 from mlflow.models.model import MLMODEL_FILE_NAME
 from mlflow.models.utils import (
     PyFuncInput,
@@ -272,28 +274,39 @@ from mlflow.utils.model_utils import (
     _get_flavor_configuration,
     _validate_and_copy_code_paths,
     _add_code_from_conf_to_system_path,
-    _get_flavor_configuration_from_uri,
+    _get_flavor_configuration_from_ml_model_file,
     _validate_and_prepare_target_save_path,
 )
 from mlflow.utils.nfs_on_spark import get_nfs_cache_root_dir
-from mlflow.utils.process import cache_return_value_per_process
 from mlflow.utils.requirements_utils import (
     _check_requirement_satisfied,
     _parse_requirements,
 )
-from mlflow.utils.uri import append_to_uri_path
 
 FLAVOR_NAME = "python_function"
 MAIN = "loader_module"
 CODE = "code"
 DATA = "data"
 ENV = "env"
+
+
+class EnvType:
+    CONDA = "conda"
+    VIRTUALENV = "virtualenv"
+
+    def __init__(self):
+        raise NotImplementedError("This class is not meant to be instantiated.")
+
+
 PY_VERSION = "python_version"
 
+
 _logger = logging.getLogger(__name__)
 
 
-def add_to_model(model, loader_module, data=None, code=None, env=None, **kwargs):
+def add_to_model(
+    model, loader_module, data=None, code=None, conda_env=None, python_env=None, **kwargs
+):
     """
     Add a ``pyfunc`` spec to the model configuration.
 
@@ -309,7 +322,8 @@ def add_to_model(model, loader_module, data=None, code=None, env=None, **kwargs)
     :param loader_module: The module to be used to load the model.
     :param data: Path to the model data.
     :param code: Path to the code dependencies.
-    :param env: Conda environment.
+    :param conda_env: Conda environment.
+    :param python_env: Python environment.
     :param req: pip requirements file.
     :param kwargs: Additional key-value pairs to include in the ``pyfunc`` flavor specification.
                    Values must be YAML-serializable.
@@ -322,12 +336,21 @@ def add_to_model(model, loader_module, data=None, code=None, env=None, **kwargs)
         params[CODE] = code
     if data:
         params[DATA] = data
-    if env:
-        params[ENV] = env
-
+    if conda_env or python_env:
+        params[ENV] = {}
+        if conda_env:
+            params[ENV][EnvType.CONDA] = conda_env
+        if python_env:
+            params[ENV][EnvType.VIRTUALENV] = python_env
     return model.add_flavor(FLAVOR_NAME, **params)
 
 
+def _extract_conda_env(env):
+    # In MLflow < 2.0.0, the 'env' field in a pyfunc configuration is a string containing the path
+    # to a conda.yaml file.
+    return env if isinstance(env, str) else env[EnvType.CONDA]
+
+
 def _load_model_env(path):
     """
     Get ENV file string from a model configuration stored in Python Function format.
@@ -372,7 +395,16 @@ class PyFuncModel:
 
         :param data: Model input as one of pandas.DataFrame, numpy.ndarray,
                      scipy.sparse.(csc.csc_matrix | csr.csr_matrix), List[Any], or
-                     Dict[str, numpy.ndarray]
+                     Dict[str, numpy.ndarray].
+                     For model signatures with tensor spec inputs
+                     (e.g. the Tensorflow core / Keras model), the input data type must be one of
+                     `numpy.ndarray`, `List[numpy.ndarray]`, `Dict[str, numpy.ndarray]` or
+                     `pandas.DataFrame`. If data is of `pandas.DataFrame` type and the model
+                     contains a signature with tensor spec inputs, the corresponding column values
+                     in the pandas DataFrame will be reshaped to the required shape with 'C' order
+                     (i.e. read / write the elements using C-like index order), and DataFrame
+                     column values will be cast as the required tensor spec type.
+
         :return: Model predictions as one of pandas.DataFrame, pandas.Series, numpy.ndarray or list.
         """
         input_schema = self.metadata.get_input_schema()
@@ -384,6 +416,49 @@ class PyFuncModel:
     def unwrap_python_model(self):
         """
         Unwrap the underlying Python model object.
+
+        This method is useful for accessing custom model functions, while still being able to
+        leverage the MLflow designed workflow through the `predict()` method.
+
+        :return: The underlying wrapped model object
+
+        .. test-code-block:: python
+            :caption: Example
+
+            import mlflow
+
+            # define a custom model
+            class MyModel(mlflow.pyfunc.PythonModel):
+                def predict(self, context, model_input):
+                    return self.my_custom_function(model_input)
+
+                def my_custom_function(self, model_input):
+                    # do something with the model input
+                    return 0
+
+            some_input = 1
+            # save the model
+            my_model = MyModel()
+            with mlflow.start_run():
+                model_info = mlflow.pyfunc.log_model(artifact_path="model", python_model=my_model)
+
+            # load the model
+            loaded_model = mlflow.pyfunc.load_model(model_uri=model_info.model_uri)
+            print(type(loaded_model)) # <class 'mlflow.pyfunc.model.PyFuncModel'>
+
+            unwrapped_model = loaded_model.unwrap_python_model()
+            print(type(unwrapped_model)) # <class '__main__.MyModel'>
+
+            # does not work, only predict() is exposed
+            # print(loaded_model.my_custom_function(some_input))
+
+            print(unwrapped_model.my_custom_function(some_input)) # works
+
+            print(loaded_model.predict(some_input)) # works
+
+            # works, but None is needed for context arg
+            print(unwrapped_model.predict(None, some_input))
+
         """
         try:
             python_model = self._model_impl.python_model
@@ -493,7 +568,7 @@ def load_model(
     conf = model_meta.flavors.get(FLAVOR_NAME)
     if conf is None:
         raise MlflowException(
-            'Model does not have the "{flavor_name}" flavor'.format(flavor_name=FLAVOR_NAME),
+            f'Model does not have the "{FLAVOR_NAME}" flavor',
             RESOURCE_DOES_NOT_EXIST,
         )
     model_py_version = conf.get(PY_VERSION)
@@ -508,15 +583,13 @@ def load_model(
 
 
 class _ServedPyFuncModel(PyFuncModel):
-    def __init__(self, model_meta: Model, client: Any, server_pid: int, env_manager: str):
+    def __init__(self, model_meta: Model, client: Any, server_pid: int):
         super().__init__(model_meta=model_meta, model_impl=client, predict_fn="invoke")
-        _EnvManager.validate(env_manager)
-        self._env_manager = env_manager
         self._client = client
         self._server_pid = server_pid
 
     def predict(self, data):
-        result = self._client.invoke(data)
+        result = self._client.invoke(data).get_predictions()
         if isinstance(result, pandas.DataFrame):
             result = result[result.columns[0]]
         return result
@@ -527,10 +600,6 @@ class _ServedPyFuncModel(PyFuncModel):
             raise MlflowException("Served PyFunc Model is missing server process ID.")
         return self._server_pid
 
-    @property
-    def env_manager(self):
-        return self._env_manager
-
 
 def _load_model_or_server(model_uri: str, env_manager: str):
     """
@@ -544,7 +613,6 @@ def _load_model_or_server(model_uri: str, env_manager: str):
     :param env_manager: The environment manager to load the model.
     :return: A _ServedPyFuncModel for non-local ``env_manager``s or a PyFuncModel otherwise.
     """
-    from mlflow.models.cli import _get_flavor_backend
     from mlflow.pyfunc.scoring_server.client import ScoringServerClient
 
     if env_manager == _EnvManager.LOCAL:
@@ -555,12 +623,11 @@ def _load_model_or_server(model_uri: str, env_manager: str):
     local_path = _download_artifact_from_uri(artifact_uri=model_uri)
     model_meta = Model.load(os.path.join(local_path, MLMODEL_FILE_NAME))
 
-    env_root_dir = _get_or_create_env_root_dir(False)
-    pyfunc_backend = _get_flavor_backend(
+    pyfunc_backend = get_flavor_backend(
         local_path,
-        install_mlflow=False,
         env_manager=env_manager,
-        env_root_dir=env_root_dir,
+        install_mlflow=os.environ.get("MLFLOW_HOME") is not None,
+        create_env_root_dir=True,
     )
     _logger.info("Restoring model environment. This can take a few minutes.")
     # Set capture_output to True in Databricks so that when environment preparation fails, the
@@ -572,7 +639,7 @@ def _load_model_or_server(model_uri: str, env_manager: str):
         model_uri=local_path,
         port=server_port,
         host="127.0.0.1",
-        timeout=60,
+        timeout=MLFLOW_SCORING_SERVER_REQUEST_TIMEOUT.get(),
         enable_mlserver=False,
         synchronous=False,
         stdout=subprocess.PIPE,
@@ -586,37 +653,34 @@ def _load_model_or_server(model_uri: str, env_manager: str):
         raise MlflowException("MLflow model server failed to launch")
 
     return _ServedPyFuncModel(
-        model_meta=model_meta,
-        client=client,
-        server_pid=scoring_server_proc.pid,
-        env_manager=env_manager,
+        model_meta=model_meta, client=client, server_pid=scoring_server_proc.pid
     )
 
 
-def _download_model_conda_env(model_uri):
-    conda_yml_file_name = _get_flavor_configuration_from_uri(model_uri, FLAVOR_NAME)[ENV]
-    return _download_artifact_from_uri(append_to_uri_path(model_uri, conda_yml_file_name))
-
-
 def _get_model_dependencies(model_uri, format="pip"):  # pylint: disable=redefined-builtin
+    model_dir = _download_artifact_from_uri(model_uri)
+
+    def get_conda_yaml_path():
+        model_config = _get_flavor_configuration_from_ml_model_file(
+            os.path.join(model_dir, MLMODEL_FILE_NAME), flavor_name=FLAVOR_NAME
+        )
+        return os.path.join(model_dir, _extract_conda_env(model_config[ENV]))
+
     if format == "pip":
-        req_file_uri = append_to_uri_path(model_uri, _REQUIREMENTS_FILE_NAME)
-        try:
-            return _download_artifact_from_uri(req_file_uri)
-        except Exception as e:
-            # fallback to download conda.yaml file and parse the "pip" section from it.
-            _logger.info(
-                f"Downloading model '{_REQUIREMENTS_FILE_NAME}' file failed, error is {repr(e)}. "
-                "Falling back to fetching pip requirements from the model's 'conda.yaml' file. "
-                "Other conda dependencies will be ignored."
-            )
+        requirements_file = os.path.join(model_dir, _REQUIREMENTS_FILE_NAME)
+        if os.path.exists(requirements_file):
+            return requirements_file
 
-        conda_yml_path = _download_model_conda_env(model_uri)
+        _logger.info(
+            f"{_REQUIREMENTS_FILE_NAME} is not found in the model directory. Falling back to"
+            f" extracting pip requirements from the model's 'conda.yaml' file. Conda"
+            " dependencies will be ignored."
+        )
 
-        with open(conda_yml_path, "r") as yf:
-            conda_yml = yaml.safe_load(yf)
+        with open(get_conda_yaml_path()) as yf:
+            conda_yaml = yaml.safe_load(yf)
 
-        conda_deps = conda_yml.get("dependencies", [])
+        conda_deps = conda_yaml.get("dependencies", [])
         for index, dep in enumerate(conda_deps):
             if isinstance(dep, dict) and "pip" in dep:
                 pip_deps_index = index
@@ -642,8 +706,7 @@ def _get_model_dependencies(model_uri, format="pip"):  # pylint: disable=redefin
         return pip_file_path
 
     elif format == "conda":
-        conda_yml_path = _download_model_conda_env(model_uri)
-        return conda_yml_path
+        return get_conda_yaml_path()
     else:
         raise MlflowException(
             f"Illegal format argument '{format}'.", error_code=INVALID_PARAMETER_VALUE
@@ -739,22 +802,10 @@ def _create_model_downloading_tmp_dir(should_use_nfs):
     return tmp_model_dir
 
 
-@cache_return_value_per_process
-def _get_or_create_env_root_dir(should_use_nfs):
-    if should_use_nfs:
-        root_tmp_dir = get_or_create_nfs_tmp_dir()
-    else:
-        root_tmp_dir = get_or_create_tmp_dir()
-
-    env_root_dir = os.path.join(root_tmp_dir, "envs")
-    os.makedirs(env_root_dir, exist_ok=True)
-    return env_root_dir
-
-
 _MLFLOW_SERVER_OUTPUT_TAIL_LINES_TO_KEEP = 200
 
 
-def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
+def spark_udf(spark, model_uri, result_type="double", env_manager=_EnvManager.LOCAL):
     """
     A Spark UDF that can be used to invoke the Python function formatted model.
 
@@ -764,6 +815,12 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
     names given by the struct definition (e.g. when invoked as my_udf(struct('x', 'y')), the model
     will get the data as a pandas DataFrame with 2 columns 'x' and 'y').
 
+    If a model contains a signature with tensor spec inputs, you will need to pass a column of
+    array type as a corresponding UDF argument. The column values of which must be one dimensional
+    arrays. The UDF will reshape the column values to the required shape with 'C' order
+    (i.e. read / write the elements using C-like index order) and cast the values as the required
+    tensor spec type.
+
     If a model contains a signature, the UDF can be called without specifying column name
     arguments. In this case, the UDF will be called with column names from signature, so the
     evaluation dataframe's column names must match the model signature's column names.
@@ -802,7 +859,8 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
 
     :param result_type: the return type of the user-defined function. The value can be either a
         ``pyspark.sql.types.DataType`` object or a DDL-formatted type string. Only a primitive
-        type or an array ``pyspark.sql.types.ArrayType`` of primitive type are allowed.
+        type, an array ``pyspark.sql.types.ArrayType`` of primitive type, or a struct type
+        containing fields of above 2 kinds of types are allowed.
         The following classes of result type are supported:
 
         - "int" or ``pyspark.sql.types.IntegerType``: The leftmost integer that can fit in an
@@ -830,16 +888,19 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
 
         - ``ArrayType(StringType)``: All columns converted to ``string``.
 
+        - "field1 FIELD1_TYPE, field2 FIELD2_TYPE, ...": A struct type containing multiple fields
+          separated by comma, each field type must be one of types listed above.
+
     :param env_manager: The environment manager to use in order to create the python environment
                         for model inference. Note that environment is only restored in the context
                         of the PySpark UDF; the software environment outside of the UDF is
                         unaffected. Default value is ``local``, and the following values are
                         supported:
 
-                         - ``conda``: (Recommended) Use Conda to restore the software environment
-                           that was used to train the model.
                          - ``virtualenv``: Use virtualenv to restore the python environment that
                            was used to train the model.
+                         - ``conda``: (Recommended) Use Conda to restore the software environment
+                           that was used to train the model.
                          - ``local``: Use the current Python environment for model inference, which
                            may differ from the environment used to train the model and may lead to
                            errors or invalid predictions.
@@ -868,7 +929,9 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
         StringType,
         BooleanType,
     )
-    from mlflow.models.cli import _get_flavor_backend
+
+    # Used in test to force install local version of mlflow when starting a model server
+    mlflow_home = os.environ.get("MLFLOW_HOME")
 
     _EnvManager.validate(env_manager)
 
@@ -879,7 +942,6 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
     nfs_root_dir = get_nfs_cache_root_dir()
     should_use_nfs = nfs_root_dir is not None
     should_use_spark_to_broadcast_file = not (is_spark_in_local_mode or should_use_nfs)
-    env_root_dir = _get_or_create_env_root_dir(should_use_nfs)
 
     result_type = "boolean" if result_type == "bool" else result_type
 
@@ -890,7 +952,15 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
     if isinstance(elem_type, ArrayType):
         elem_type = elem_type.elementType
 
-    supported_types = [IntegerType, LongType, FloatType, DoubleType, StringType, BooleanType]
+    supported_types = [
+        IntegerType,
+        LongType,
+        FloatType,
+        DoubleType,
+        StringType,
+        BooleanType,
+        SparkStructType,
+    ]
 
     if not any(isinstance(elem_type, x) for x in supported_types):
         raise MlflowException(
@@ -929,7 +999,12 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
                 "limitations with handling SIGKILL signals, these MLflow Model server child "
                 "processes cannot be cleaned up if the Spark Job is canceled."
             )
-
+    pyfunc_backend = get_flavor_backend(
+        local_model_path,
+        env_manager=env_manager,
+        install_mlflow=os.environ.get("MLFLOW_HOME") is not None,
+        create_env_root_dir=True,
+    )
     if not should_use_spark_to_broadcast_file:
         # Prepare restored environment in driver side if possible.
         # Note: In databricks runtime, because databricks notebook cell output cannot capture
@@ -941,15 +1016,11 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
         # to wait conda command fail and suddenly get all output printed (included in error
         # message).
         if env_manager != _EnvManager.LOCAL:
-            _get_flavor_backend(
-                local_model_path,
-                env_manager=env_manager,
-                install_mlflow=False,
-                env_root_dir=env_root_dir,
-            ).prepare_env(model_uri=local_model_path, capture_output=is_in_databricks_runtime())
-
-    # Broadcast local model directory to remote worker if needed.
-    if should_use_spark_to_broadcast_file:
+            pyfunc_backend.prepare_env(
+                model_uri=local_model_path, capture_output=is_in_databricks_runtime()
+            )
+    else:
+        # Broadcast local model directory to remote worker if needed.
         archive_path = SparkModelCache.add_local_model(spark, local_model_path)
 
     model_metadata = Model.load(os.path.join(local_model_path, MLMODEL_FILE_NAME))
@@ -976,8 +1047,8 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
                     args = args[: len(names)]
                 if len(args) < len(names):
                     raise MlflowException(
-                        "Model input is missing columns. Expected {0} input columns {1},"
-                        " but the model received only {2} unnamed input columns"
+                        "Model input is missing columns. Expected {} input columns {},"
+                        " but the model received only {} unnamed input columns"
                         " (Since the columns were passed unnamed they are expected to be in"
                         " the order specified by the schema).".format(len(names), names, len(args))
                     )
@@ -985,17 +1056,66 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
 
         result = predict_fn(pdf)
 
+        if isinstance(result, dict):
+            result = {k: list(v) for k, v in result.items()}
+
         if not isinstance(result, pandas.DataFrame):
             result = pandas.DataFrame(data=result)
 
+        spark_primitive_type_to_np_type = {
+            IntegerType: np.int32,
+            LongType: np.int64,
+            FloatType: np.float32,
+            DoubleType: np.float64,
+            BooleanType: np.bool_,
+            StringType: np.str_,
+        }
+
+        if isinstance(result_type, SparkStructType):
+            result_dict = {}
+            for field_name in result_type.fieldNames():
+                field_type = result_type[field_name].dataType
+                field_values = result[field_name]
+
+                if type(field_type) in spark_primitive_type_to_np_type:
+                    np_type = spark_primitive_type_to_np_type[type(field_type)]
+                    field_values = field_values.astype(np_type)
+
+                elif type(field_type) == ArrayType:
+                    elem_type = field_type.elementType
+                    if type(elem_type) not in spark_primitive_type_to_np_type:
+                        raise MlflowException(
+                            "Unsupported array type field with element type "
+                            f"{elem_type.simpleString()} in struct type.",
+                            error_code=INVALID_PARAMETER_VALUE,
+                        )
+                    np_type = spark_primitive_type_to_np_type[type(elem_type)]
+
+                    field_values = [
+                        np.array(v, dtype=np.dtype(type(elem_type))) for v in field_values
+                    ]
+
+                else:
+                    raise MlflowException(
+                        f"Unsupported field type {field_type.simpleString()} in struct type.",
+                        error_code=INVALID_PARAMETER_VALUE,
+                    )
+
+                result_dict[field_name] = field_values
+
+            return pandas.DataFrame(result_dict)
+
         elem_type = result_type.elementType if isinstance(result_type, ArrayType) else result_type
 
         if type(elem_type) == IntegerType:
             result = result.select_dtypes(
                 [np.byte, np.ubyte, np.short, np.ushort, np.int32]
             ).astype(np.int32)
+
         elif type(elem_type) == LongType:
-            result = result.select_dtypes([np.byte, np.ubyte, np.short, np.ushort, int])
+            result = result.select_dtypes([np.byte, np.ubyte, np.short, np.ushort, int]).astype(
+                np.int64
+            )
 
         elif type(elem_type) == FloatType:
             result = result.select_dtypes(include=(np.number,)).astype(np.float32)
@@ -1035,7 +1155,8 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
 
         # Note: this is a pandas udf function in iteration style, which takes an iterator of
         # tuple of pandas.Series and outputs an iterator of pandas.Series.
-
+        if mlflow_home is not None:
+            os.environ["MLFLOW_HOME"] = mlflow_home
         scoring_server_proc = None
 
         if env_manager != _EnvManager.LOCAL:
@@ -1043,21 +1164,6 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
                 local_model_path_on_executor = _SparkDirectoryDistributor.get_or_extract(
                     archive_path
                 )
-                # Create individual conda_env_root_dir for each spark UDF task process.
-                env_root_dir_on_executor = _get_or_create_env_root_dir(should_use_nfs)
-            else:
-                local_model_path_on_executor = local_model_path
-                env_root_dir_on_executor = env_root_dir
-
-            pyfunc_backend = _get_flavor_backend(
-                local_model_path_on_executor,
-                workers=1,
-                install_mlflow=False,
-                env_manager=env_manager,
-                env_root_dir=env_root_dir_on_executor,
-            )
-
-            if should_use_spark_to_broadcast_file:
                 # Call "prepare_env" in advance in order to reduce scoring server launch time.
                 # So that we can use a shorter timeout when call `client.wait_server_ready`,
                 # otherwise we have to set a long timeout for `client.wait_server_ready` time,
@@ -1069,14 +1175,15 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
                 pyfunc_backend.prepare_env(
                     model_uri=local_model_path_on_executor, capture_output=True
                 )
-
+            else:
+                local_model_path_on_executor = None
             # launch scoring server
             server_port = find_free_port()
             scoring_server_proc = pyfunc_backend.serve(
-                model_uri=local_model_path_on_executor,
+                model_uri=local_model_path_on_executor or local_model_path,
                 port=server_port,
                 host="127.0.0.1",
-                timeout=60,
+                timeout=MLFLOW_SCORING_SERVER_REQUEST_TIMEOUT.get(),
                 enable_mlserver=False,
                 synchronous=False,
                 stdout=subprocess.PIPE,
@@ -1118,7 +1225,7 @@ def spark_udf(spark, model_uri, result_type="double", env_manager="local"):
                 raise MlflowException(err_msg)
 
             def batch_predict_fn(pdf):
-                return client.invoke(pdf)
+                return client.invoke(pdf).get_predictions()
 
         elif env_manager == _EnvManager.LOCAL:
             if should_use_spark_to_broadcast_file:
@@ -1195,6 +1302,7 @@ def save_model(
     input_example: ModelInputExample = None,
     pip_requirements=None,
     extra_pip_requirements=None,
+    metadata=None,
     **kwargs,
 ):
     """
@@ -1280,12 +1388,16 @@ def save_model(
                           by converting it to a list. Bytes are base64-encoded.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     """
     _validate_env_arguments(conda_env, pip_requirements, extra_pip_requirements)
 
     mlflow_model = kwargs.pop("model", mlflow_model)
     if len(kwargs) > 0:
-        raise TypeError("save_model() got unexpected keyword arguments: {}".format(kwargs))
+        raise TypeError(f"save_model() got unexpected keyword arguments: {kwargs}")
     if code_path is not None:
         if not isinstance(code_path, list):
             raise TypeError("Argument code_path should be a list, not {}".format(type(code_path)))
@@ -1328,6 +1440,8 @@ def save_model(
         mlflow_model.signature = signature
     if input_example is not None:
         _save_example(mlflow_model, input_example, path)
+    if metadata is not None:
+        mlflow_model.metadata = metadata
 
     if first_argument_set_specified:
         return _save_model_with_loader_module_and_data_path(
@@ -1368,6 +1482,7 @@ def log_model(
     await_registration_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
     pip_requirements=None,
     extra_pip_requirements=None,
+    metadata=None,
 ):
     """
     Log a Pyfunc model with custom inference logic and optional data dependencies as an MLflow
@@ -1453,6 +1568,10 @@ def log_model(
                             waits for five minutes. Specify 0 or None to skip waiting.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     :return: A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
              metadata of the logged model.
     """
@@ -1471,6 +1590,7 @@ def log_model(
         await_registration_for=await_registration_for,
         pip_requirements=pip_requirements,
         extra_pip_requirements=extra_pip_requirements,
+        metadata=metadata,
     )
 
 
@@ -1516,7 +1636,8 @@ def _save_model_with_loader_module_and_data_path(
         loader_module=loader_module,
         code=code_dir_subpath,
         data=data,
-        env=_CONDA_ENV_FILE_NAME,
+        conda_env=_CONDA_ENV_FILE_NAME,
+        python_env=_PYTHON_ENV_FILE_NAME,
     )
     mlflow_model.save(os.path.join(path, MLMODEL_FILE_NAME))
 
