@@ -1,9 +1,14 @@
 import requests
 import time
 import json
+import numpy as np
+import pandas as pd
 
 from mlflow.pyfunc import scoring_server
-from mlflow.utils.proto_json_utils import _DateTimeEncoder
+
+from mlflow.exceptions import MlflowException
+from mlflow.utils.proto_json_utils import _CustomJsonEncoder
+from mlflow.deployments import PredictionsResponse
 
 
 class ScoringServerClient:
@@ -41,23 +46,34 @@ class ScoringServerClient:
 
     def invoke(self, data):
         """
-        Invoke inference on input data. The input data must be pandas dataframe or json instance.
+        Invoke inference on input data. The input data must be pandas dataframe or numpy array or
+        a dict of numpy arrays.
         """
         content_type = scoring_server.CONTENT_TYPE_JSON
-        post_data = json.dumps(
-            scoring_server._get_jsonable_obj(data, pandas_orient="split"),
-            cls=_DateTimeEncoder,
-        )
 
+        def get_jsonable_input(name, data):
+            if isinstance(data, np.ndarray):
+                return data.tolist()
+            else:
+                raise MlflowException(f"Incompatible input type:{type(data)} for input {name}.")
+
+        if isinstance(data, pd.DataFrame):
+            post_data = {"dataframe_split": data.to_dict(orient="split")}
+        elif isinstance(data, dict):
+            post_data = {"inputs": {k: get_jsonable_input(k, v) for k, v in data}}
+        elif isinstance(data, np.ndarray):
+            post_data = ({"inputs": data.tolist()},)
+        else:
+            post_data = data
+        if not isinstance(post_data, str):
+            post_data = json.dumps(post_data, cls=_CustomJsonEncoder)
         response = requests.post(
             url=self.url_prefix + "/invocations",
             data=post_data,
             headers={"Content-Type": content_type},
         )
-
         if response.status_code != 200:
             raise Exception(
                 f"Invocation failed (error code {response.status_code}, response: {response.text})"
             )
-
-        return scoring_server.infer_and_parse_json_input(response.text)
+        return PredictionsResponse.from_json(response.text)
