@@ -28,7 +28,6 @@ from mlflow.models.utils import ModelInputExample, _save_example
 from mlflow.protos.databricks_pb2 import RESOURCE_DOES_NOT_EXIST
 from mlflow.pytorch import pickle_module as mlflow_pytorch_pickle_module
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
-from mlflow.utils.annotations import experimental
 from mlflow.utils.environment import (
     _mlflow_conda_env,
     _validate_env_arguments,
@@ -109,7 +108,7 @@ def get_default_conda_env():
 
         conda env {'name': 'mlflow-env',
                    'channels': ['conda-forge'],
-                   'dependencies': ['python=3.7.5',
+                   'dependencies': ['python=3.8.15',
                                     {'pip': ['torch==1.5.1',
                                              'mlflow',
                                              'cloudpickle==1.6.0']}]}
@@ -132,6 +131,7 @@ def log_model(
     extra_files=None,
     pip_requirements=None,
     extra_pip_requirements=None,
+    metadata=None,
     **kwargs,
 ):
     """
@@ -226,6 +226,10 @@ def log_model(
                       If ``None``, no extra files are added to the model.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     :param kwargs: kwargs to pass to ``torch.save`` method.
     :return: A :py:class:`ModelInfo <mlflow.models.model.ModelInfo>` instance that contains the
              metadata of the logged model.
@@ -318,6 +322,7 @@ def log_model(
         extra_files=extra_files,
         pip_requirements=pip_requirements,
         extra_pip_requirements=extra_pip_requirements,
+        metadata=metadata,
         **kwargs,
     )
 
@@ -336,6 +341,7 @@ def save_model(
     extra_files=None,
     pip_requirements=None,
     extra_pip_requirements=None,
+    metadata=None,
     **kwargs,
 ):
     """
@@ -415,6 +421,10 @@ def save_model(
                       If ``None``, no extra files are added to the model.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     :param kwargs: kwargs to pass to ``torch.save`` method.
 
     .. code-block:: python
@@ -487,6 +497,8 @@ def save_model(
         mlflow_model.signature = signature
     if input_example is not None:
         _save_example(mlflow_model, input_example, path)
+    if metadata is not None:
+        mlflow_model.metadata = metadata
 
     code_dir_subpath = _validate_and_copy_code_paths(code_paths, path)
 
@@ -562,7 +574,8 @@ def save_model(
         data=model_data_subpath,
         pickle_module_name=pickle_module.__name__,
         code=code_dir_subpath,
-        env=_CONDA_ENV_FILE_NAME,
+        conda_env=_CONDA_ENV_FILE_NAME,
+        python_env=_PYTHON_ENV_FILE_NAME,
     )
     mlflow_model.save(os.path.join(path, MLMODEL_FILE_NAME))
 
@@ -613,7 +626,7 @@ def _load_model(path, **kwargs):
         # information about the pickle module that should be used by PyTorch to load it
         model_path = os.path.join(path, "model.pth")
         pickle_module_path = os.path.join(path, _PICKLE_MODULE_INFO_FILE_NAME)
-        with open(pickle_module_path, "r") as f:
+        with open(pickle_module_path) as f:
             pickle_module_name = f.read()
         if "pickle_module" in kwargs and kwargs["pickle_module"].__name__ != pickle_module_name:
             _logger.warning(
@@ -629,9 +642,7 @@ def _load_model(path, **kwargs):
                 raise MlflowException(
                     message=(
                         "Failed to import the pickle module that was used to save the PyTorch"
-                        " model. Pickle module name: `{pickle_module_name}`".format(
-                            pickle_module_name=pickle_module_name
-                        )
+                        f" model. Pickle module name: `{pickle_module_name}`"
                     ),
                     error_code=RESOURCE_DOES_NOT_EXIST,
                 ) from exc
@@ -777,7 +788,6 @@ class _PyTorchWrapper:
             return predicted
 
 
-@experimental
 def log_state_dict(state_dict, artifact_path, **kwargs):
     """
     Log a state_dict as an MLflow artifact for the current run.
@@ -815,7 +825,6 @@ def log_state_dict(state_dict, artifact_path, **kwargs):
         mlflow.log_artifacts(local_path, artifact_path)
 
 
-@experimental
 def save_state_dict(state_dict, path, **kwargs):
     """
     Save a state_dict to a path on the local file system
@@ -841,7 +850,6 @@ def save_state_dict(state_dict, path, **kwargs):
     torch.save(state_dict, state_dict_path, **kwargs)
 
 
-@experimental
 def load_state_dict(state_dict_uri, **kwargs):
     """
     Load a state_dict from a local file or a run.
