@@ -13,6 +13,7 @@ from mlflow.server.handlers import (
     get_model_version_artifact_handler,
 )
 from mlflow.utils.process import _exec_cmd
+from mlflow.version import VERSION
 
 # NB: These are internal environment variables used for communication between
 # the cli and the forked gunicorn processes.
@@ -48,6 +49,12 @@ def health():
     return "OK", 200
 
 
+# Provide an endpoint to query the version of mlflow running on the server
+@app.route("/version")
+def version():
+    return VERSION, 200
+
+
 # Serve the "get-artifact" route.
 @app.route(_add_static_prefix("/get-artifact"))
 def serve_artifacts():
@@ -82,7 +89,7 @@ def serve():
 
     If you are a developer making MLflow source code changes and intentionally running a source
     installation of MLflow, you can view the UI by running the Javascript dev server:
-    https://github.com/mlflow/mlflow/blob/master/CONTRIBUTING.rst#running-the-javascript-dev-server
+    https://github.com/mlflow/mlflow/blob/master/CONTRIBUTING.md#running-the-javascript-dev-server
 
     Otherwise, uninstall MLflow via 'pip uninstall mlflow', reinstall an official MLflow release
     from PyPI via 'pip install mlflow', and rerun the MLflow server.
@@ -101,7 +108,7 @@ def _build_waitress_command(waitress_opts, host, port):
 
 
 def _build_gunicorn_command(gunicorn_opts, host, port, workers):
-    bind_address = "%s:%s" % (host, port)
+    bind_address = f"{host}:{port}"
     opts = shlex.split(gunicorn_opts) if gunicorn_opts else []
     return ["gunicorn"] + opts + ["-b", bind_address, "-w", "%s" % workers, "mlflow.server:app"]
 
