@@ -144,7 +144,7 @@ def _log_matplotlib_figure(fig, out_file, artifact_path=None):
 
 def _get_conda_env_for_underlying_model(underlying_model_path):
     underlying_model_conda_path = os.path.join(underlying_model_path, "conda.yaml")
-    with open(underlying_model_conda_path, "r") as underlying_model_conda_file:
+    with open(underlying_model_conda_path) as underlying_model_conda_file:
         return yaml.safe_load(underlying_model_conda_file)
 
 
@@ -300,6 +300,7 @@ def log_explainer(
     await_registration_for=DEFAULT_AWAIT_MAX_SLEEP_SECONDS,
     pip_requirements=None,
     extra_pip_requirements=None,
+    metadata=None,
 ):
     """
     Log an SHAP explainer as an MLflow artifact for the current run.
@@ -343,6 +344,10 @@ def log_explainer(
                             waits for five minutes. Specify 0 or None to skip waiting.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     """
 
     Model.log(
@@ -358,6 +363,7 @@ def log_explainer(
         await_registration_for=await_registration_for,
         pip_requirements=pip_requirements,
         extra_pip_requirements=extra_pip_requirements,
+        metadata=metadata,
     )
 
 
@@ -373,6 +379,7 @@ def save_explainer(
     input_example: ModelInputExample = None,
     pip_requirements=None,
     extra_pip_requirements=None,
+    metadata=None,
 ):
     """
     Save a SHAP explainer to a path on the local file system. Produces an MLflow Model
@@ -414,6 +421,10 @@ def save_explainer(
                           base64-encoded.
     :param pip_requirements: {{ pip_requirements }}
     :param extra_pip_requirements: {{ extra_pip_requirements }}
+    :param metadata: Custom metadata dictionary passed to the model and stored in the MLmodel file.
+
+                     .. Note:: Experimental: This parameter may change or be removed in a future
+                                             release without warning.
     """
     import shap
 
@@ -428,6 +439,8 @@ def save_explainer(
         mlflow_model.signature = signature
     if input_example is not None:
         _save_example(mlflow_model, input_example, path)
+    if metadata is not None:
+        mlflow_model.metadata = metadata
 
     underlying_model_flavor = None
     underlying_model_path = None
