@@ -8,15 +8,15 @@ import urllib.parse
 
 from mlflow import data
 from mlflow.entities import FileInfo
-from mlflow.exceptions import MlflowException
-from mlflow.store.artifact.artifact_repo import ArtifactRepository
-from mlflow.utils.file_utils import relative_path_to_artifact_path
-
 from mlflow.environment_variables import (
     MLFLOW_S3_UPLOAD_EXTRA_ARGS,
     MLFLOW_S3_ENDPOINT_URL,
     MLFLOW_S3_IGNORE_TLS,
 )
+from mlflow.exceptions import MlflowException
+from mlflow.store.artifact.artifact_repo import ArtifactRepository
+from mlflow.utils.file_utils import relative_path_to_artifact_path
+
 
 _MAX_CACHE_SECONDS = 300
 
@@ -183,10 +183,8 @@ class S3ArtifactRepository(ArtifactRepository):
         if not listed_object_path.startswith(artifact_path):
             raise MlflowException(
                 "The path of the listed S3 object does not begin with the specified"
-                " artifact path. Artifact path: {artifact_path}. Object path:"
-                " {object_path}.".format(
-                    artifact_path=artifact_path, object_path=listed_object_path
-                )
+                f" artifact path. Artifact path: {artifact_path}. Object path:"
+                f" {listed_object_path}."
             )
 
     def _download_file(self, remote_file_path, local_path):
