@@ -7,16 +7,30 @@ import logging
 from alembic.migration import MigrationContext  # pylint: disable=import-error
 from alembic.script import ScriptDirectory
 import sqlalchemy
+from sqlalchemy import sql
+
+# We need to import sqlalchemy.pool to convert poolclass string to class object
+from sqlalchemy.pool import (
+    AssertionPool,
+    AsyncAdaptedQueuePool,
+    FallbackAsyncAdaptedQueuePool,
+    NullPool,
+    QueuePool,
+    SingletonThreadPool,
+    StaticPool,
+)
+
 
 from mlflow.exceptions import MlflowException
 from mlflow.store.tracking.dbmodels.initial_models import Base as InitialBase
-from mlflow.protos.databricks_pb2 import BAD_REQUEST, INTERNAL_ERROR
+from mlflow.protos.databricks_pb2 import BAD_REQUEST, INTERNAL_ERROR, TEMPORARILY_UNAVAILABLE
 from mlflow.store.db.db_types import SQLITE
 from mlflow.environment_variables import (
     MLFLOW_SQLALCHEMYSTORE_POOL_SIZE,
     MLFLOW_SQLALCHEMYSTORE_POOL_RECYCLE,
     MLFLOW_SQLALCHEMYSTORE_MAX_OVERFLOW,
     MLFLOW_SQLALCHEMYSTORE_ECHO,
+    MLFLOW_SQLALCHEMYSTORE_POOLCLASS,
 )
 
 _logger = logging.getLogger(__name__)
@@ -78,14 +92,21 @@ def _get_managed_session_maker(SessionMaker, db_type):
         session = SessionMaker()
         try:
             if db_type == SQLITE:
-                session.execute("PRAGMA foreign_keys = ON;")
-                session.execute("PRAGMA busy_timeout = 20000;")
-                session.execute("PRAGMA case_sensitive_like = true;")
+                session.execute(sql.text("PRAGMA foreign_keys = ON;"))
+                session.execute(sql.text("PRAGMA busy_timeout = 20000;"))
+                session.execute(sql.text("PRAGMA case_sensitive_like = true;"))
             yield session
             session.commit()
         except MlflowException:
             session.rollback()
             raise
+        except sqlalchemy.exc.OperationalError as e:
+            session.rollback()
+            _logger.exception(
+                "SQLAlchemy database error. The following exception is caught.\n%s",
+                e,
+            )
+            raise MlflowException(message=e, error_code=TEMPORARILY_UNAVAILABLE)
         except sqlalchemy.exc.SQLAlchemyError as e:
             session.rollback()
             raise MlflowException(message=e, error_code=BAD_REQUEST)
@@ -186,6 +207,7 @@ def create_sqlalchemy_engine(db_uri):
     pool_max_overflow = MLFLOW_SQLALCHEMYSTORE_MAX_OVERFLOW.get()
     pool_recycle = MLFLOW_SQLALCHEMYSTORE_POOL_RECYCLE.get()
     echo = MLFLOW_SQLALCHEMYSTORE_ECHO.get()
+    poolclass = MLFLOW_SQLALCHEMYSTORE_POOLCLASS.get()
     pool_kwargs = {}
     # Send argument only if they have been injected.
     # Some engine does not support them (for example sqllite)
@@ -197,6 +219,25 @@ def create_sqlalchemy_engine(db_uri):
         pool_kwargs["pool_recycle"] = pool_recycle
     if echo:
         pool_kwargs["echo"] = echo
+    if poolclass:
+        pool_class_map = {
+            "AssertionPool": AssertionPool,
+            "AsyncAdaptedQueuePool": AsyncAdaptedQueuePool,
+            "FallbackAsyncAdaptedQueuePool": FallbackAsyncAdaptedQueuePool,
+            "NullPool": NullPool,
+            "QueuePool": QueuePool,
+            "SingletonThreadPool": SingletonThreadPool,
+            "StaticPool": StaticPool,
+        }
+        if poolclass not in pool_class_map:
+            list_str = " ".join(pool_class_map.keys())
+            err_str = (
+                f"Invalid poolclass parameter: {poolclass}. Set environment variable "
+                f"poolclass to empty or one of the following values: {list_str}"
+            )
+            _logger.warning(err_str)
+            raise ValueError(err_str)
+        pool_kwargs["poolclass"] = pool_class_map[poolclass]
     if pool_kwargs:
         _logger.info("Create SQLAlchemy engine with pool options %s", pool_kwargs)
     return sqlalchemy.create_engine(db_uri, pool_pre_ping=True, **pool_kwargs)
