@@ -31,7 +31,7 @@ from mlflow.store.model_registry.dbmodels.models import (
     SqlRegisteredModelTag,
     SqlModelVersionTag,
 )
-from mlflow.utils.search_utils import SearchUtils, SearchModelUtils
+from mlflow.utils.search_utils import SearchUtils, SearchModelUtils, SearchModelVersionUtils
 from mlflow.utils.uri import extract_db_type_from_uri
 from mlflow.utils.validation import (
     _validate_registered_model_tag,
@@ -209,7 +209,7 @@ class SqlAlchemyStore(AbstractStore):
 
         if len(rms) == 0:
             raise MlflowException(
-                "Registered Model with name={} not found".format(name), RESOURCE_DOES_NOT_EXIST
+                f"Registered Model with name={name} not found", RESOURCE_DOES_NOT_EXIST
             )
         if len(rms) > 1:
             raise MlflowException(
@@ -300,10 +300,8 @@ class SqlAlchemyStore(AbstractStore):
         """
         if max_results > SEARCH_REGISTERED_MODEL_MAX_RESULTS_THRESHOLD:
             raise MlflowException(
-                "Invalid value for request parameter max_results. "
-                "It must be at most {}, but got value {}".format(
-                    SEARCH_REGISTERED_MODEL_MAX_RESULTS_THRESHOLD, max_results
-                ),
+                "Invalid value for request parameter max_results. It must be at most "
+                f"{SEARCH_REGISTERED_MODEL_MAX_RESULTS_THRESHOLD}, but got value {max_results}",
                 INVALID_PARAMETER_VALUE,
             )
 
@@ -383,7 +381,7 @@ class SqlAlchemyStore(AbstractStore):
 
         rm_query = select(SqlRegisteredModel).filter(*attribute_filters)
         if tag_filters:
-            sql_tag_filters = map(lambda x: sqlalchemy.and_(*x), tag_filters.values())
+            sql_tag_filters = (sqlalchemy.and_(*x) for x in tag_filters.values())
             tag_filter_query = (
                 select(SqlRegisteredModelTag.name)
                 .filter(sqlalchemy.or_(*sql_tag_filters))
@@ -453,7 +451,7 @@ class SqlAlchemyStore(AbstractStore):
 
         mv_query = select(SqlModelVersion).filter(*attribute_filters)
         if tag_filters:
-            sql_tag_filters = map(lambda x: sqlalchemy.and_(*x), tag_filters.values())
+            sql_tag_filters = (sqlalchemy.and_(*x) for x in tag_filters.values())
             tag_filter_query = (
                 select(SqlModelVersionTag.name, SqlModelVersionTag.version)
                 .filter(sqlalchemy.or_(*sql_tag_filters))
@@ -490,15 +488,13 @@ class SqlAlchemyStore(AbstractStore):
                     field = SqlRegisteredModel.last_updated_time
                 else:
                     raise MlflowException(
-                        "Invalid order by key '{}' specified.".format(attribute_token)
+                        f"Invalid order by key '{attribute_token}' specified."
                         + "Valid keys are "
-                        + "'{}'".format(SearchUtils.RECOMMENDED_ORDER_BY_KEYS_REGISTERED_MODELS),
+                        + f"'{SearchUtils.RECOMMENDED_ORDER_BY_KEYS_REGISTERED_MODELS}'",
                         error_code=INVALID_PARAMETER_VALUE,
                     )
                 if field.key in observed_order_by_clauses:
-                    raise MlflowException(
-                        "`order_by` contains duplicate fields: {}".format(order_by_list)
-                    )
+                    raise MlflowException(f"`order_by` contains duplicate fields: {order_by_list}")
                 observed_order_by_clauses.add(field.key)
                 if ascending:
                     clauses.append(field.asc())
@@ -651,8 +647,8 @@ class SqlAlchemyStore(AbstractStore):
                         "s" if more_retries > 1 else "",
                     )
         raise MlflowException(
-            "Model Version creation error (name={}). Giving up after "
-            "{} attempts.".format(name, self.CREATE_MODEL_VERSION_RETRIES)
+            f"Model Version creation error (name={name}). Giving up after "
+            f"{self.CREATE_MODEL_VERSION_RETRIES} attempts."
         )
 
     @classmethod
@@ -663,13 +659,13 @@ class SqlAlchemyStore(AbstractStore):
 
         if len(versions) == 0:
             raise MlflowException(
-                "Model Version (name={}, version={}) not found".format(name, version),
+                f"Model Version (name={name}, version={version}) not found",
                 RESOURCE_DOES_NOT_EXIST,
             )
         if len(versions) > 1:
             raise MlflowException(
-                "Expected only 1 model version with (name={}, version={}). "
-                "Found {}.".format(name, version, len(versions)),
+                f"Expected only 1 model version with (name={name}, version={version}). "
+                f"Found {len(versions)}.",
                 INVALID_STATE,
             )
         return versions[0]
@@ -731,7 +727,7 @@ class SqlAlchemyStore(AbstractStore):
 
         :param name: Registered model name.
         :param version: Registered model version.
-        :param new_stage: New desired stage for this model version.
+        :param stage: New desired stage for this model version.
         :param archive_existing_versions: If this flag is set to ``True``, all existing model
             versions in the stage will be automatically moved to the "archived" stage. Only valid
             when ``stage`` is ``"staging"`` or ``"production"`` otherwise an error will be raised.
@@ -831,7 +827,7 @@ class SqlAlchemyStore(AbstractStore):
         :return: PagedList of :py:class:`mlflow.entities.model_registry.ModelVersion`
                  objects sorted by last updated time in descending order.
         """
-        parsed_filters = SearchModelUtils.parse_search_filter(filter_string)
+        parsed_filters = SearchModelVersionUtils.parse_search_filter(filter_string)
 
         filter_query = self._get_search_model_versions_filter_clauses(
             parsed_filters, self.engine.dialect.name
