@@ -9,12 +9,13 @@ from mlflow.environment_variables import MLFLOW_TRACKING_AWS_SIGV4
 from mlflow.store.tracking import DEFAULT_LOCAL_FILE_AND_ARTIFACT_PATH
 from mlflow.store.db.db_types import DATABASE_ENGINES
 from mlflow.store.tracking.file_store import FileStore
-from mlflow.store.tracking.rest_store import RestStore, DatabricksRestStore
+from mlflow.store.tracking.rest_store import RestStore
 from mlflow.tracking._tracking_service.registry import TrackingStoreRegistry
 from mlflow.utils import env, rest_utils
 from mlflow.utils.file_utils import path_to_local_file_uri
 from mlflow.utils.databricks_utils import get_databricks_host_creds
 
+
 _TRACKING_URI_ENV_VAR = "MLFLOW_TRACKING_URI"
 
 # Extra environment variables which take precedence for setting the basic/bearer
@@ -59,7 +60,7 @@ def set_tracking_uri(uri: Union[str, Path]) -> None:
                   "databricks://<profileName>".
                 - A :py:class:`pathlib.Path` instance
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -74,7 +75,7 @@ def set_tracking_uri(uri: Union[str, Path]) -> None:
         Current tracking uri: file:///tmp/my_tracking
     """
     if isinstance(uri, Path):
-        # On Windows with Python3.7 (https://bugs.python.org/issue38671)
+        # On Windows with Python3.8 (https://bugs.python.org/issue38671)
         # .resolve() doesn't return the absolute path if the directory doesn't exist
         # so we're calling .absolute() first to get the absolute path on Windows,
         # then .resolve() to clean the path
@@ -167,7 +168,7 @@ def _get_rest_store(store_uri, **_):
 
 
 def _get_databricks_rest_store(store_uri, **_):
-    return DatabricksRestStore(partial(get_databricks_host_creds, store_uri))
+    return RestStore(partial(get_databricks_host_creds, store_uri))
 
 
 _tracking_store_registry = TrackingStoreRegistry()
