@@ -8,10 +8,9 @@ import atexit
 import logging
 import inspect
 from copy import deepcopy
-from packaging.version import Version
 from typing import Any, Dict, List, Optional, Union, TYPE_CHECKING
 
-from mlflow.entities import Experiment, Run, RunInfo, RunStatus, Param, RunTag, Metric, ViewType
+from mlflow.entities import Experiment, Run, RunStatus, Param, RunTag, Metric, ViewType
 from mlflow.entities.lifecycle_stage import LifecycleStage
 from mlflow.exceptions import MlflowException
 from mlflow.protos.databricks_pb2 import (
@@ -23,8 +22,7 @@ from mlflow.tracking import artifact_utils, _get_store
 from mlflow.tracking.context import registry as context_registry
 from mlflow.tracking.default_experiment import registry as default_experiment_registry
 from mlflow.store.tracking import SEARCH_MAX_RESULTS_DEFAULT
-from mlflow.utils import env
-from mlflow.utils.annotations import deprecated
+from mlflow.utils import env, get_results_from_paginated_fn
 from mlflow.utils.autologging_utils import (
     is_testing,
     autologging_integration,
@@ -79,7 +77,7 @@ def set_experiment(experiment_name: str = None, experiment_id: str = None) -> Ex
     :return: An instance of :py:class:`mlflow.entities.Experiment` representing the new active
              experiment.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -215,7 +213,7 @@ def start_run(
     :return: :py:class:`mlflow.ActiveRun` object that acts as a context manager wrapping
              the run's state.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -293,17 +291,15 @@ def start_run(
             and _active_experiment_id != active_run_obj.info.experiment_id
         ):
             raise MlflowException(
-                "Cannot start run with ID {} because active run ID "
+                f"Cannot start run with ID {existing_run_id} because active run ID "
                 "does not match environment run ID. Make sure --experiment-name "
                 "or --experiment-id matches experiment set with "
-                "set_experiment(), or just use command-line "
-                "arguments".format(existing_run_id)
+                "set_experiment(), or just use command-line arguments"
             )
         # Check to see if current run isn't deleted
         if active_run_obj.info.lifecycle_stage == LifecycleStage.DELETED:
             raise MlflowException(
-                "Cannot start run with ID {} because it is in the "
-                "deleted state.".format(existing_run_id)
+                f"Cannot start run with ID {existing_run_id} because it is in the deleted state."
             )
         # Use previous end_time because a value is required for update_run_info
         end_time = active_run_obj.info.end_time
@@ -361,7 +357,7 @@ def start_run(
 def end_run(status: str = RunStatus.to_string(RunStatus.FINISHED)) -> None:
     """End an active MLflow run (if there is one).
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -407,7 +403,7 @@ def active_run() -> Optional[ActiveRun]:
     (parameters, metrics, etc.) through the run returned by ``mlflow.active_run``. In order
     to access such attributes, use the :py:class:`mlflow.client.MlflowClient` as follows:
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -431,7 +427,7 @@ def last_active_run() -> Optional[Run]:
 
     Examples:
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: To retrieve the most recent autologged run:
 
         import mlflow
@@ -453,7 +449,7 @@ def last_active_run() -> Optional[Run]:
         predictions = rf.predict(X_test)
         autolog_run = mlflow.last_active_run()
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: To get the most recently active run that ended:
 
         import mlflow
@@ -462,7 +458,7 @@ def last_active_run() -> Optional[Run]:
         mlflow.end_run()
         run = mlflow.last_active_run()
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: To retrieve the currently active run:
 
         import mlflow
@@ -497,7 +493,7 @@ def get_run(run_id: str) -> Run:
     :return: A single :py:class:`mlflow.entities.Run` object, if the run exists. Otherwise,
                 raises an exception.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -532,7 +528,7 @@ def log_param(key: str, value: Any) -> Any:
 
     :return: the parameter value that is logged.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -557,7 +553,7 @@ def set_experiment_tag(key: str, value: Any) -> None:
                   All backend stores will support values up to length 5000, but some
                   may support larger values.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -582,7 +578,7 @@ def set_tag(key: str, value: Any) -> None:
                   All backend stores will support values up to length 5000, but some
                   may support larger values.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -601,7 +597,7 @@ def delete_tag(key: str) -> None:
 
     :param key: Name of the tag
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -635,7 +631,7 @@ def log_metric(key: str, value: float, step: Optional[int] = None) -> None:
                   may support larger values.
     :param step: Metric step (int). Defaults to zero if unspecified.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -661,7 +657,7 @@ def log_metrics(metrics: Dict[str, float], step: Optional[int] = None) -> None:
 
     :returns: None
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -687,7 +683,7 @@ def log_params(params: Dict[str, Any]) -> None:
                    not)
     :returns: None
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -709,7 +705,7 @@ def set_experiment_tags(tags: Dict[str, Any]) -> None:
 
     :param tags: Dictionary containing tag names and corresponding values.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -735,7 +731,7 @@ def set_tags(tags: Dict[str, Any]) -> None:
                  not)
     :returns: None
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -761,7 +757,7 @@ def log_artifact(local_path: str, artifact_path: Optional[str] = None) -> None:
     :param local_path: Path to the file to write.
     :param artifact_path: If provided, the directory in ``artifact_uri`` to write to.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -788,9 +784,10 @@ def log_artifacts(local_dir: str, artifact_path: Optional[str] = None) -> None:
     :param local_dir: Path to the directory of files to write.
     :param artifact_path: If provided, the directory in ``artifact_uri`` to write to.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
+        import json
         import os
         import mlflow
 
@@ -821,7 +818,7 @@ def log_text(text: str, artifact_file: str) -> None:
     :param artifact_file: The run-relative artifact file path in posixpath format to which
                           the text is saved (e.g. "dir/file.txt").
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -851,7 +848,7 @@ def log_dict(dictionary: Any, artifact_file: str) -> None:
     :param artifact_file: The run-relative artifact file path in posixpath format to which
                           the dictionary is saved (e.g. "dir/data.json").
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -893,7 +890,7 @@ def log_figure(
     :param artifact_file: The run-relative artifact file path in posixpath format to which
                           the figure is saved (e.g. "dir/file.png").
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Matplotlib Example
 
         import mlflow
@@ -905,7 +902,7 @@ def log_figure(
         with mlflow.start_run():
             mlflow.log_figure(fig, "figure.png")
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Plotly Example
 
         import mlflow
@@ -957,7 +954,7 @@ def log_image(image: Union["numpy.ndarray", "PIL.Image.Image"], artifact_file: s
     :param artifact_file: The run-relative artifact file path in posixpath format to which
                           the image is saved (e.g. "dir/image.png").
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Numpy Example
 
         import mlflow
@@ -968,7 +965,7 @@ def log_image(image: Union["numpy.ndarray", "PIL.Image.Image"], artifact_file: s
         with mlflow.start_run():
             mlflow.log_image(image, "image.png")
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Pillow Example
 
         import mlflow
@@ -995,7 +992,7 @@ def get_experiment(experiment_id: str) -> Experiment:
     :param experiment_id: The string-ified experiment ID returned from ``create_experiment``.
     :return: :py:class:`mlflow.entities.Experiment`
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -1027,7 +1024,7 @@ def get_experiment_by_name(name: str) -> Optional[Experiment]:
     :return: An instance of :py:class:`mlflow.entities.Experiment`
              if an experiment with the specified name exists, otherwise None.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -1052,28 +1049,6 @@ def get_experiment_by_name(name: str) -> Optional[Experiment]:
     return MlflowClient().get_experiment_by_name(name)
 
 
-@deprecated(alternative="search_experiments()")
-def list_experiments(
-    view_type: int = ViewType.ACTIVE_ONLY,
-    max_results: Optional[int] = None,
-) -> List[Experiment]:
-    """
-    :param view_type: Qualify requested type of experiments.
-    :param max_results: If passed, specifies the maximum number of experiments desired. If not
-                        passed, all experiments will be returned.
-    :return: A list of :py:class:`Experiment <mlflow.entities.Experiment>` objects.
-    """
-
-    def pagination_wrapper_func(number_to_get, next_page_token):
-        return MlflowClient().list_experiments(
-            view_type=view_type,
-            max_results=number_to_get,
-            page_token=next_page_token,
-        )
-
-    return _paginate(pagination_wrapper_func, SEARCH_MAX_RESULTS_DEFAULT, max_results)
-
-
 def search_experiments(
     view_type: int = ViewType.ACTIVE_ONLY,
     max_results: Optional[int] = None,
@@ -1128,7 +1103,7 @@ def search_experiments(
 
     :return: A list of :py:class:`Experiment <mlflow.entities.Experiment>` objects.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -1184,7 +1159,7 @@ def search_experiments(
             page_token=next_page_token,
         )
 
-    return _paginate(
+    return get_results_from_paginated_fn(
         pagination_wrapper_func,
         SEARCH_MAX_RESULTS_DEFAULT,
         max_results,
@@ -1206,7 +1181,7 @@ def create_experiment(
                             tags on the experiment.
     :return: String ID of the created experiment.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -1245,7 +1220,7 @@ def delete_experiment(experiment_id: str) -> None:
 
     :param experiment_id: The The string-ified experiment ID returned from ``create_experiment``.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -1276,7 +1251,7 @@ def delete_run(run_id: str) -> None:
 
     :param run_id: Unique identifier for the run to delete.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -1317,7 +1292,7 @@ def get_artifact_uri(artifact_path: Optional[str] = None) -> str:
              is not provided and the currently active run uses an S3-backed store, this may be a
              URI of the form ``s3://<bucket_name>/path/to/artifact/root``.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -1392,7 +1367,7 @@ def search_runs(
              the value for the corresponding column is (NumPy) ``Nan``, ``None``, or ``None``
              respectively.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import mlflow
@@ -1449,7 +1424,7 @@ def search_runs(
 
     if search_all_experiments and no_ids_or_names:
         experiment_ids = [
-            exp.experiment_id for exp in list_experiments(view_type=ViewType.ACTIVE_ONLY)
+            exp.experiment_id for exp in search_experiments(view_type=ViewType.ACTIVE_ONLY)
         ]
     elif no_ids_or_names:
         experiment_ids = _get_experiment_id()
@@ -1469,7 +1444,11 @@ def search_runs(
             next_page_token,
         )
 
-    runs = _paginate(pagination_wrapper_func, NUM_RUNS_PER_PAGE_PANDAS, max_results)
+    runs = get_results_from_paginated_fn(
+        pagination_wrapper_func,
+        NUM_RUNS_PER_PAGE_PANDAS,
+        max_results,
+    )
 
     if output_format == "list":
         return runs  # List[mlflow.entities.run.Run]
@@ -1547,110 +1526,6 @@ def search_runs(
         )
 
 
-@deprecated(alternative="search_runs()")
-def list_run_infos(
-    experiment_id: str,
-    run_view_type: int = ViewType.ACTIVE_ONLY,
-    max_results: int = SEARCH_MAX_RESULTS_DEFAULT,
-    order_by: Optional[List[str]] = None,
-) -> List[RunInfo]:
-    """
-    Return run information for runs which belong to the experiment_id.
-
-    :param experiment_id: The experiment id which to search
-    :param run_view_type: ACTIVE_ONLY, DELETED_ONLY, or ALL runs
-    :param max_results: Maximum number of results desired.
-    :param order_by: List of order_by clauses. Currently supported values are
-           are ``metric.key``, ``parameter.key``, ``tag.key``, ``attribute.key``.
-           For example, ``order_by=["tag.release ASC", "metric.click_rate DESC"]``.
-
-    :return: A list of :py:class:`RunInfo <mlflow.entities.RunInfo>` objects that satisfy the
-        search expressions.
-
-    .. code-block:: python
-        :caption: Example
-
-        import mlflow
-        from mlflow.entities import ViewType
-
-        # Create two runs
-        with mlflow.start_run() as run1:
-            mlflow.log_param("p", 0)
-
-        with mlflow.start_run() as run2:
-            mlflow.log_param("p", 1)
-
-        # Delete the last run
-        mlflow.delete_run(run2.info.run_id)
-
-        def print_run_infos(run_infos):
-            for r in run_infos:
-                print("- run_id: {}, lifecycle_stage: {}".format(r.run_id, r.lifecycle_stage))
-
-        print("Active runs:")
-        print_run_infos(mlflow.list_run_infos("0", run_view_type=ViewType.ACTIVE_ONLY))
-
-        print("Deleted runs:")
-        print_run_infos(mlflow.list_run_infos("0", run_view_type=ViewType.DELETED_ONLY))
-
-        print("All runs:")
-        print_run_infos(mlflow.list_run_infos("0", run_view_type=ViewType.ALL))
-
-    .. code-block:: text
-        :caption: Output
-
-        Active runs:
-        - run_id: 4937823b730640d5bed9e3e5057a2b34, lifecycle_stage: active
-        Deleted runs:
-        - run_id: b13f1badbed842cf9975c023d23da300, lifecycle_stage: deleted
-        All runs:
-        - run_id: b13f1badbed842cf9975c023d23da300, lifecycle_stage: deleted
-        - run_id: 4937823b730640d5bed9e3e5057a2b34, lifecycle_stage: active
-    """
-
-    # Using an internal function as the linter doesn't like assigning a lambda, and inlining the
-    # full thing is a mess
-    def pagination_wrapper_func(number_to_get, next_page_token):
-        return MlflowClient().list_run_infos(
-            experiment_id, run_view_type, number_to_get, order_by, next_page_token
-        )
-
-    return _paginate(pagination_wrapper_func, SEARCH_MAX_RESULTS_DEFAULT, max_results)
-
-
-def _paginate(paginated_fn, max_results_per_page, max_results=None):
-    """
-    Intended to be a general use pagination utility.
-
-    :param paginated_fn:
-    :type paginated_fn: This function is expected to take in the number of results to retrieve
-        per page and a pagination token, and return a PagedList object
-    :param max_results_per_page:
-    :type max_results_per_page: The maximum number of results to retrieve per page
-    :param max_results:
-    :type max_results: The maximum number of results to retrieve overall. If unspecified,
-                       all results will be retrieved.
-    :return: Returns a list of entities, as determined by the paginated_fn parameter, with no more
-        entities than specified by max_results
-    :rtype: list[object]
-    """
-    all_results = []
-    next_page_token = None
-    returns_all = max_results is None
-    while returns_all or len(all_results) < max_results:
-        num_to_get = max_results_per_page if returns_all else max_results - len(all_results)
-        if num_to_get < max_results_per_page:
-            page_results = paginated_fn(num_to_get, next_page_token)
-        else:
-            page_results = paginated_fn(max_results_per_page, next_page_token)
-        all_results.extend(page_results)
-        if hasattr(page_results, "token") and page_results.token:
-            next_page_token = page_results.token
-        else:
-            break
-    return all_results
-
-
 def _get_or_start_run():
     if len(_active_run_stack) > 0:
         return _active_run_stack[-1]
@@ -1659,18 +1534,40 @@ def _get_or_start_run():
 
 def _get_experiment_id_from_env():
     experiment_name = env.get_env(_EXPERIMENT_NAME_ENV_VAR)
+    experiment_id = env.get_env(_EXPERIMENT_ID_ENV_VAR)
     if experiment_name is not None:
         exp = MlflowClient().get_experiment_by_name(experiment_name)
-        return exp.experiment_id if exp else None
-    return env.get_env(_EXPERIMENT_ID_ENV_VAR)
+        if exp:
+            if experiment_id and experiment_id != exp.experiment_id:
+                raise MlflowException(
+                    message=f"The provided {_EXPERIMENT_ID_ENV_VAR} environment variable "
+                    f"value `{experiment_id}` does not match the experiment id "
+                    f"`{exp.experiment_id}` for experiment name `{experiment_name}`",
+                    error_code=INVALID_PARAMETER_VALUE,
+                )
+            else:
+                return exp.experiment_id
+        else:
+            experiment_id = MlflowClient().create_experiment(name=experiment_name)
+            return experiment_id
+    if experiment_id is not None:
+        try:
+            exp = MlflowClient().get_experiment(experiment_id)
+            return exp.experiment_id
+        except MlflowException as exc:
+            raise MlflowException(
+                message=f"The provided {_EXPERIMENT_ID_ENV_VAR} environment variable "
+                f"value `{experiment_id}` does not exist in the tracking server. Provide a valid "
+                f"experiment_id.",
+                error_code=INVALID_PARAMETER_VALUE,
+            ) from exc
 
 
 def _get_experiment_id():
-    return (
-        _active_experiment_id
-        or _get_experiment_id_from_env()
-        or default_experiment_registry.get_experiment_id()
-    )
+    if _active_experiment_id:
+        return _active_experiment_id
+    else:
+        return _get_experiment_id_from_env() or default_experiment_registry.get_experiment_id()
 
 
 @autologging_integration("mlflow")
@@ -1695,17 +1592,20 @@ def autolog(
     Note that framework-specific configurations set at any point will take precedence over
     any configurations set by this function. For example:
 
-    .. code-block:: python
+    .. test-code-block:: python
 
+        import mlflow
         mlflow.autolog(log_models=False, exclusive=True)
         import sklearn
 
     would enable autologging for `sklearn` with `log_models=False` and `exclusive=True`,
     but
 
-    .. code-block:: python
+    .. test-code-block:: python
 
+        import mlflow
         mlflow.autolog(log_models=False, exclusive=True)
+
         import sklearn
         mlflow.sklearn.autolog(log_models=True)
 
@@ -1742,7 +1642,7 @@ def autolog(
                    setup and training execution. If ``False``, show all events and warnings during
                    autologging setup and training execution.
 
-    .. code-block:: python
+    .. test-code-block:: python
         :caption: Example
 
         import numpy as np
@@ -1782,16 +1682,15 @@ def autolog(
                  'fit_intercept': 'True',
                  'n_jobs': 'None'}
         metrics: {'training_score': 1.0,
-                  'training_rmse': 4.440892098500626e-16,
+                  'training_root_mean_squared_error': 4.440892098500626e-16,
                   'training_r2_score': 1.0,
-                  'training_mae': 2.220446049250313e-16,
-                  'training_mse': 1.9721522630525295e-31}
+                  'training_mean_absolute_error': 2.220446049250313e-16,
+                  'training_mean_squared_error': 1.9721522630525295e-31}
         tags: {'estimator_class': 'sklearn.linear_model._base.LinearRegression',
                'estimator_name': 'LinearRegression'}
     """
     from mlflow import (
         tensorflow,
-        keras,
         gluon,
         xgboost,
         lightgbm,
@@ -1809,7 +1708,6 @@ def autolog(
     # eg: mxnet.gluon is the actual library, mlflow.gluon.autolog is our autolog function for it
     LIBRARY_TO_AUTOLOG_FN = {
         "tensorflow": tensorflow.autolog,
-        "keras": keras.autolog,
         "mxnet.gluon": gluon.autolog,
         "xgboost": xgboost.autolog,
         "lightgbm": lightgbm.autolog,
@@ -1875,64 +1773,9 @@ def autolog(
     # for each autolog library (except pyspark), register a post-import hook.
     # this way, we do not send any errors to the user until we know they are using the library.
     # the post-import hook also retroactively activates for previously-imported libraries.
-    for module in list(
-        set(LIBRARY_TO_AUTOLOG_FN.keys()) - {"tensorflow", "keras", "pyspark", "pyspark.ml"}
-    ):
+    for module in list(set(LIBRARY_TO_AUTOLOG_FN.keys()) - {"pyspark", "pyspark.ml"}):
         register_post_import_hook(setup_autologging, module, overwrite=True)
 
-    FULLY_IMPORTED_KERAS = False
-    TF_AUTOLOG_SETUP_CALLED = False
-
-    def conditionally_set_up_keras_autologging(keras_module):
-        nonlocal FULLY_IMPORTED_KERAS, TF_AUTOLOG_SETUP_CALLED
-        FULLY_IMPORTED_KERAS = True
-
-        if Version(keras_module.__version__) >= Version("2.6.0"):
-            # NB: Keras unconditionally depends on TensorFlow beginning with Version 2.6.0, and
-            # many classes defined in the `keras` module are aliases of classes in the `tf.keras`
-            # module. Accordingly, TensorFlow autologging serves as a replacement for Keras
-            # autologging in Keras >= 2.6.0
-            try:
-                import tensorflow
-
-                setup_autologging(tensorflow)
-                TF_AUTOLOG_SETUP_CALLED = True
-            except Exception as e:
-                _logger.debug(
-                    "Failed to set up TensorFlow autologging for tf.keras models upon"
-                    " Keras library import: %s",
-                    str(e),
-                )
-                raise
-        else:
-            setup_autologging(keras_module)
-
-    register_post_import_hook(conditionally_set_up_keras_autologging, "keras", overwrite=True)
-
-    def set_up_tensorflow_autologging(tensorflow_module):
-        import sys
-
-        nonlocal FULLY_IMPORTED_KERAS, TF_AUTOLOG_SETUP_CALLED
-        if "keras" in sys.modules and not FULLY_IMPORTED_KERAS:
-            # In Keras >= 2.6.0, importing Keras imports the TensorFlow library, which can
-            # trigger this autologging import hook for TensorFlow before the entire Keras import
-            # procedure is completed. Attempting to set up autologging before the Keras import
-            # procedure has completed will result in a failure due to the unavailability of
-            # certain modules. In this case, we terminate the TensorFlow autologging import hook
-            # and rely on the Keras autologging import hook to successfully set up TensorFlow
-            # autologging for tf.keras models once the Keras import procedure has completed
-            return
-
-        # By design, in Keras >= 2.6.0, Keras needs to enable tensorflow autologging so that
-        # tf.keras models always use tensorflow autologging, rather than vanilla keras autologging.
-        # As a result, Keras autologging must call `mlflow.tensorflow.autolog()` in Keras >= 2.6.0.
-        # Accordingly, we insert this check to ensure that importing tensorflow, which may import
-        # keras, does not enable tensorflow autologging twice.
-        if not TF_AUTOLOG_SETUP_CALLED:
-            setup_autologging(tensorflow_module)
-
-    register_post_import_hook(set_up_tensorflow_autologging, "tensorflow", overwrite=True)
-
     # for pyspark, we activate autologging immediately, without waiting for a module import.
     # this is because on Databricks a SparkSession already exists and the user can directly
     #   interact with it, and this activity should be logged.
