@@ -40,10 +40,9 @@ def get_unique_resource_id(max_length=None):
     # unsupported '+' symbol maintains uniqueness because the UUID byte string is of a fixed,
     # 16-byte length
     uuid_b64 = base64.b64encode(uuid_bytes)
-    if version_info >= (3, 0):
-        # In Python3, `uuid_b64` is a `bytes` object. It needs to be
-        # converted to a string
-        uuid_b64 = uuid_b64.decode("ascii")
+    # In Python3, `uuid_b64` is a `bytes` object. It needs to be
+    # converted to a string
+    uuid_b64 = uuid_b64.decode("ascii")
     unique_id = uuid_b64.rstrip("=\n").replace("/", "-").replace("+", "AB").lower()
     if max_length is not None:
         unique_id = unique_id[: int(max_length)]
@@ -109,13 +108,13 @@ def _truncate_dict(d, max_key_length=None, max_value_length=None):
         new_k = _truncate_and_ellipsize(k, max_key_length) if should_truncate_key else k
         if should_truncate_key:
             # Use the truncated key for warning logs to avoid noisy printing to stdout
-            msg = "Truncated the key `{}`".format(new_k)
+            msg = f"Truncated the key `{new_k}`"
             _logger.warning(msg)
 
         new_v = _truncate_and_ellipsize(v, max_value_length) if should_truncate_val else v
         if should_truncate_val:
             # Use the truncated key and value for warning logs to avoid noisy printing to stdout
-            msg = "Truncated the value of the key `{}`. Truncated value: `{}`".format(new_k, new_v)
+            msg = f"Truncated the value of the key `{new_k}`. Truncated value: `{new_v}`"
             _logger.warning(msg)
 
         truncated[new_k] = new_v
@@ -219,3 +218,37 @@ def _is_in_ipython_notebook():
         return get_ipython() is not None
     except Exception:
         return False
+
+
+def get_results_from_paginated_fn(paginated_fn, max_results_per_page, max_results=None):
+    """
+    Gets results by calling the ``paginated_fn`` until either no more results remain or
+    the specified ``max_results`` threshold has been reached.
+
+    :param paginated_fn:
+    :type paginated_fn: This function is expected to take in the number of results to retrieve
+        per page and a pagination token, and return a PagedList object
+    :param max_results_per_page:
+    :type max_results_per_page: The maximum number of results to retrieve per page
+    :param max_results:
+    :type max_results: The maximum number of results to retrieve overall. If unspecified,
+                       all results will be retrieved.
+    :return: Returns a list of entities, as determined by the paginated_fn parameter, with no more
+        entities than specified by max_results
+    :rtype: list[object]
+    """
+    all_results = []
+    next_page_token = None
+    returns_all = max_results is None
+    while returns_all or len(all_results) < max_results:
+        num_to_get = max_results_per_page if returns_all else max_results - len(all_results)
+        if num_to_get < max_results_per_page:
+            page_results = paginated_fn(num_to_get, next_page_token)
+        else:
+            page_results = paginated_fn(max_results_per_page, next_page_token)
+        all_results.extend(page_results)
+        if hasattr(page_results, "token") and page_results.token:
+            next_page_token = page_results.token
+        else:
+            break
+    return all_results
