@@ -16,16 +16,15 @@ from mlflow.utils.validation import MAX_METRICS_PER_BATCH
 _logger = logging.getLogger(__name__)
 
 # Import autologging utilities used by this module
-from mlflow.utils.autologging_utils.logging_and_warnings import (  # noqa: E402
+from mlflow.utils.autologging_utils.logging_and_warnings import (
     set_mlflow_events_and_warnings_behavior_globally,
     set_non_mlflow_warnings_behavior_for_current_thread,
 )
-from mlflow.utils.autologging_utils.safety import (  # noqa: E402
-    try_mlflow_log,
+from mlflow.utils.autologging_utils.safety import (
     update_wrapper_extended,
     revert_patches,
 )
-from mlflow.utils.autologging_utils.versioning import (  # noqa: E402
+from mlflow.utils.autologging_utils.versioning import (
     FLAVOR_TO_MODULE_NAME_AND_VERSION_INFO_KEY,
     get_min_max_version_and_pip_release,
     is_flavor_supported_for_associated_package_versions,
@@ -34,9 +33,9 @@ from mlflow.utils.autologging_utils.versioning import (  # noqa: E402
 # Wildcard import other autologging utilities (e.g. safety utilities, event logging utilities) used
 # in autologging integration implementations, which reference them via the
 # `mlflow.utils.autologging_utils` module
-from mlflow.utils.autologging_utils.safety import *  # noqa: E402
-from mlflow.utils.autologging_utils.events import *  # noqa: E402
-from mlflow.utils.autologging_utils.client import *  # noqa: E402
+from mlflow.utils.autologging_utils.safety import *
+from mlflow.utils.autologging_utils.events import *
+from mlflow.utils.autologging_utils.client import *
 
 
 INPUT_EXAMPLE_SAMPLE_ROWS = 5
@@ -48,6 +47,12 @@ _AUTOLOGGING_TEST_MODE_ENV_VAR = "MLFLOW_AUTOLOGGING_TESTING"
 # Flag indicating whether autologging is globally disabled for all integrations.
 _AUTOLOGGING_GLOBALLY_DISABLED = False
 
+# Autologging config key indicating whether or not a particular autologging integration
+# was configured (i.e. its various `log_models`, `disable`, etc. configuration options
+# were set) via a call to `mlflow.autolog()`, rather than via a call to the integration-specific
+# autologging method (e.g., `mlflow.tensorflow.autolog()`, ...)
+AUTOLOGGING_CONF_KEY_IS_GLOBALLY_CONFIGURED = "globally_configured"
+
 # Dict mapping integration name to its config.
 AUTOLOGGING_INTEGRATIONS = {}
 
@@ -194,8 +199,9 @@ class BatchMetricsLogger:
     `record_metrics()` or `flush()`.
     """
 
-    def __init__(self, run_id=None):
+    def __init__(self, run_id=None, tracking_uri=None):
         self.run_id = run_id
+        self.client = MlflowClient(tracking_uri)
 
         # data is an array of Metric objects
         self.data = []
@@ -223,7 +229,7 @@ class BatchMetricsLogger:
             for i in range(0, len(self.data), MAX_METRICS_PER_BATCH)
         ]
         for metrics_slice in metrics_slices:
-            try_mlflow_log(MlflowClient().log_batch, run_id=current_run_id, metrics=metrics_slice)
+            self.client.log_batch(run_id=current_run_id, metrics=metrics_slice)
         end = time.time()
         self.total_log_batch_time += end - start
 
@@ -297,9 +303,7 @@ def gen_autologging_package_version_requirements_doc(integration_name):
     """
     _, module_key = FLAVOR_TO_MODULE_NAME_AND_VERSION_INFO_KEY[integration_name]
     min_ver, max_ver, pip_release = get_min_max_version_and_pip_release(module_key)
-    required_pkg_versions = "``{min_ver}`` <= ``{pip_release}`` <= ``{max_ver}``".format(
-        min_ver=min_ver, pip_release=pip_release, max_ver=max_ver
-    )
+    required_pkg_versions = f"``{min_ver}`` <= ``{pip_release}`` <= ``{max_ver}``"
 
     return (
         "    .. Note:: Autologging is known to be compatible with the following package versions: "
@@ -343,13 +347,13 @@ def autologging_integration(name):
     def validate_param_spec(param_spec):
         if "disable" not in param_spec or param_spec["disable"].default is not False:
             raise Exception(
-                "Invalid `autolog()` function for integration '{}'. `autolog()` functions"
-                " must specify a 'disable' argument with default value 'False'".format(name)
+                f"Invalid `autolog()` function for integration '{name}'. `autolog()` functions"
+                " must specify a 'disable' argument with default value 'False'"
             )
         elif "silent" not in param_spec or param_spec["silent"].default is not False:
             raise Exception(
-                "Invalid `autolog()` function for integration '{}'. `autolog()` functions"
-                " must specify a 'silent' argument with default value 'False'".format(name)
+                f"Invalid `autolog()` function for integration '{name}'. `autolog()` functions"
+                " must specify a 'silent' argument with default value 'False'"
             )
 
     def wrapper(_autolog):
@@ -376,11 +380,12 @@ def autologging_integration(name):
             except Exception:
                 pass
 
+            revert_patches(name)
+
             # If disabling autologging using fluent api, then every active integration's autolog
             # needs to be called with disable=True. So do not short circuit and let
             # `mlflow.autolog()` invoke all active integrations with disable=True.
             if name != "mlflow" and get_autologging_config(name, "disable", True):
-                revert_patches(name)
                 return
 
             is_silent_mode = get_autologging_config(name, "silent", False)
@@ -504,19 +509,19 @@ def _get_new_training_session_class():
     # 1. We don't currently have any use cases for allow_children=True.
     # 2. The list append & pop operations are thread-safe, so we will always clear the session stack
     #    once all _TrainingSessions exit.
-    class _TrainingSession(object):
+    class _TrainingSession:
         _session_stack = []
 
-        def __init__(self, clazz, allow_children=True):
+        def __init__(self, estimator, allow_children=True):
             """
             A session manager for nested autologging runs.
 
-            :param clazz: A class object that this session originates from.
+            :param estimator: An estimator that this session originates from.
             :param allow_children: If True, allows autologging in child sessions.
                                    If False, disallows autologging in all descendant sessions.
             """
             self.allow_children = allow_children
-            self.clazz = clazz
+            self.estimator = estimator
             self._parent = None
 
         def __enter__(self):
@@ -536,11 +541,61 @@ def _get_new_training_session_class():
             Returns True when at least one of the following conditions satisfies:
 
             1. This session is the root session.
-            2. The parent session allows autologging and its class differs from this session's
-               class.
+            2. The parent session allows autologging and its estimator differs from this session's
+               estimator.
             """
-            return (self._parent is None) or (
-                self._parent.allow_children and self._parent.clazz != self.clazz
-            )
+            for training_session in _TrainingSession._session_stack:
+                if training_session is self:
+                    break
+                elif training_session.estimator is self.estimator:
+                    return False
+
+            return self._parent is None or self._parent.allow_children
+
+        @staticmethod
+        def is_active():
+            return len(_TrainingSession._session_stack) != 0
+
+        @staticmethod
+        def get_current_session():
+            if _TrainingSession.is_active():
+                return _TrainingSession._session_stack[-1]
+            return None
 
     return _TrainingSession
+
+
+def get_instance_method_first_arg_value(method, call_pos_args, call_kwargs):
+    """
+    Get instance method first argument value (exclude the `self` argument).
+    :param method A `cls.method` object which includes the `self` argument.
+    :param call_pos_args: positional arguments excluding the first `self` argument.
+    :param call_kwargs: keywords arguments.
+    """
+    if len(call_pos_args) >= 1:
+        return call_pos_args[0]
+    else:
+        param_sig = inspect.signature(method).parameters
+        first_arg_name = list(param_sig.keys())[1]
+        assert param_sig[first_arg_name].kind not in [
+            inspect.Parameter.VAR_KEYWORD,
+            inspect.Parameter.VAR_POSITIONAL,
+        ]
+        return call_kwargs.get(first_arg_name)
+
+
+def get_method_call_arg_value(arg_index, arg_name, default_value, call_pos_args, call_kwargs):
+    """
+    Get argument value for a method call.
+    :param arg_index: the argument index in the function signature. start from 0.
+    :param arg_name: the argument name in the function signature.
+    :param default_value: default argument value.
+    :param call_pos_args: the positional argument values in the method call.
+    :param call_kwargs: the keyword argument values in the method call.
+    """
+    if arg_name in call_kwargs:
+        return call_kwargs[arg_name]
+    elif arg_index < len(call_pos_args):
+        return call_pos_args[arg_index]
+    else:
+        return default_value
