@@ -6,6 +6,7 @@ This module provides a set of utilities for interpreting and creating requiremen
 import json
 import sys
 import subprocess
+from threading import Timer
 import tempfile
 import os
 import pkg_resources
@@ -18,6 +19,7 @@ from typing import NamedTuple, Optional
 from pathlib import Path
 
 import mlflow
+from mlflow.environment_variables import MLFLOW_REQUIREMENTS_INFERENCE_TIMEOUT
 from mlflow.exceptions import MlflowException
 from mlflow.tracking.artifact_utils import _download_artifact_from_uri
 from mlflow.utils.autologging_utils.versioning import _strip_dev_version_suffix
@@ -198,24 +200,30 @@ def _prune_packages(packages):
     return packages - requires
 
 
-def _run_command(cmd):
+def _run_command(cmd, timeout_seconds, env=None):
     """
     Runs the specified command. If it exits with non-zero status, `MlflowException` is raised.
     """
-    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-    stdout, stderr = proc.communicate()
-    stdout = stdout.decode("utf-8")
-    stderr = stderr.decode("utf-8")
-    if proc.returncode != 0:
-        msg = "\n".join(
-            [
-                f"Encountered an unexpected error while running {cmd}",
-                f"exit status: {proc.returncode}",
-                f"stdout: {stdout}",
-                f"stderr: {stderr}",
-            ]
-        )
-        raise MlflowException(msg)
+    proc = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env)
+    timer = Timer(timeout_seconds, proc.kill)
+    try:
+        timer.start()
+        stdout, stderr = proc.communicate()
+        stdout = stdout.decode("utf-8")
+        stderr = stderr.decode("utf-8")
+        if proc.returncode != 0:
+            msg = "\n".join(
+                [
+                    f"Encountered an unexpected error while running {cmd}",
+                    f"exit status: {proc.returncode}",
+                    f"stdout: {stdout}",
+                    f"stderr: {stderr}",
+                ]
+            )
+            raise MlflowException(msg)
+    finally:
+        if timer.is_alive():
+            timer.cancel()
 
 
 def _get_installed_version(package, module=None):
@@ -260,9 +268,19 @@ def _capture_imported_modules(model_uri, flavor):
 
     local_model_path = _download_artifact_from_uri(model_uri)
 
+    process_timeout = MLFLOW_REQUIREMENTS_INFERENCE_TIMEOUT.get()
+
     # Run `_capture_modules.py` to capture modules imported during the loading procedure
     with tempfile.TemporaryDirectory() as tmpdir:
         output_file = os.path.join(tmpdir, "imported_modules.txt")
+        # Pass the main environment variables to the subprocess for environment variable mapping
+        main_env = os.environ.copy()
+        # Reset the path variable from the main process so that the subprocess retains all
+        # main process configuration that a user has.
+        # See: ``https://github.com/mlflow/mlflow/issues/6905`` for context on minio configuration
+        # resolution in a subprocess based on PATH entries.
+        main_env["PATH"] = "/usr/sbin:/sbin:" + main_env["PATH"]
+
         _run_command(
             [
                 sys.executable,
@@ -276,6 +294,8 @@ def _capture_imported_modules(model_uri, flavor):
                 "--sys-path",
                 json.dumps(sys.path),
             ],
+            timeout_seconds=process_timeout,
+            env=main_env,
         )
         with open(output_file) as f:
             return f.read().splitlines()
@@ -332,7 +352,7 @@ _PyPIPackageIndex = namedtuple("_PyPIPackageIndex", ["date", "package_names"])
 
 def _load_pypi_package_index():
     pypi_index_path = pkg_resources.resource_filename(mlflow.__name__, "pypi_package_index.json")
-    with open(pypi_index_path, "r") as f:
+    with open(pypi_index_path) as f:
         index_dict = json.load(f)
 
     return _PyPIPackageIndex(
@@ -436,16 +456,11 @@ def _get_pinned_requirement(package, version=None, module=None):
         if local_version_label:
             version = _strip_local_version_label(version_raw)
             msg = (
-                "Found {package} version ({version_raw}) contains a local version label "
-                "(+{local_version_label}). MLflow logged a pip requirement for this package as "
-                "'{package}=={version_logged}' without the local version label to make it "
+                f"Found {package} version ({version_raw}) contains a local version label "
+                f"(+{local_version_label}). MLflow logged a pip requirement for this package as "
+                f"'{package}=={version}' without the local version label to make it "
                 "installable from PyPI. To specify pip requirements containing local version "
                 "labels, please use `conda_env` or `pip_requirements`."
-            ).format(
-                package=package,
-                version_raw=version_raw,
-                version_logged=version,
-                local_version_label=local_version_label,
             )
             _logger.warning(msg)
 
