@@ -6,7 +6,10 @@ import re
 from pathlib import Path
 from packaging.version import Version
 
+import mlflow
 from mlflow.exceptions import MlflowException
+from mlflow.models.model import Model, MLMODEL_FILE_NAME
+from mlflow.utils.file_utils import TempDir
 from mlflow.utils.process import _exec_cmd, _join_commands, _IS_UNIX
 from mlflow.utils.requirements_utils import _parse_requirements
 from mlflow.utils.environment import (
@@ -100,7 +103,7 @@ def _find_latest_installable_python_version(version_prefix):
     semantic_versions = filter(_SEMANTIC_VERSION_REGEX.match, map(str.strip, lines))
     matched = [v for v in semantic_versions if v.startswith(version_prefix)]
     if not matched:
-        raise MlflowException((f"Could not find python version that matches {version_prefix}"))
+        raise MlflowException(f"Could not find python version that matches {version_prefix}")
     return sorted(matched, key=Version)[-1]
 
 
@@ -140,13 +143,38 @@ def _install_python(version, pyenv_root=None, capture_output=False):
         path_to_bin = ("bin", "python")
     else:
         # pyenv-win doesn't provide the `pyenv root` command
-        pyenv_root = pyenv_root or os.getenv("PYENV_ROOT")
+        pyenv_root = os.getenv("PYENV_ROOT")
         if pyenv_root is None:
             raise MlflowException("Environment variable 'PYENV_ROOT' must be set")
         path_to_bin = ("python.exe",)
     return Path(pyenv_root).joinpath("versions", version, *path_to_bin)
 
 
+def _get_conda_env_file(model_config):
+    from mlflow.pyfunc import _extract_conda_env
+
+    for flavor, config in model_config.flavors.items():
+        if flavor == mlflow.pyfunc.FLAVOR_NAME:
+            env = config.get(mlflow.pyfunc.ENV)
+            if env:
+                return _extract_conda_env(env)
+    return _CONDA_ENV_FILE_NAME
+
+
+def _get_python_env_file(model_config):
+    from mlflow.pyfunc import EnvType
+
+    for flavor, config in model_config.flavors.items():
+        if flavor == mlflow.pyfunc.FLAVOR_NAME:
+            env = config.get(mlflow.pyfunc.ENV)
+            if isinstance(env, dict):
+                # Models saved in MLflow >= 2.0 use a dictionary for the pyfunc flavor
+                # `env` config, where the keys are different environment managers (e.g.
+                # conda, virtualenv) and the values are corresponding environment paths
+                return env[EnvType.VIRTUALENV]
+    return _PYTHON_ENV_FILE_NAME
+
+
 def _get_python_env(local_model_path):
     """
     Constructs `_PythonEnv` from the model artifacts stored in `local_model_path`. If
@@ -158,9 +186,11 @@ def _get_python_env(local_model_path):
     :param local_model_path: Local directory containing the model artifacts.
     :return: `_PythonEnv` instance.
     """
-    python_env_file = local_model_path / _PYTHON_ENV_FILE_NAME
+    model_config = Model.load(local_model_path / MLMODEL_FILE_NAME)
+    python_env_file = local_model_path / _get_python_env_file(model_config)
+    conda_env_file = local_model_path / _get_conda_env_file(model_config)
     requirements_file = local_model_path / _REQUIREMENTS_FILE_NAME
-    conda_env_file = local_model_path / _CONDA_ENV_FILE_NAME
+
     if python_env_file.exists():
         return _PythonEnv.from_yaml(python_env_file)
     else:
@@ -190,7 +220,7 @@ def _get_virtualenv_name(python_env, work_dir_path, env_id=None):
         base_dir=work_dir_path,
     )
     return _get_mlflow_env_name(
-        str(python_env) + "".join(map(lambda x: x.req_str, requirements)) + (env_id or "")
+        str(python_env) + "".join(x.req_str for x in requirements) + (env_id or "")
     )
 
 
@@ -211,19 +241,57 @@ def _create_virtualenv(
 
     _logger.info("Installing dependencies")
     for deps in filter(None, [python_env.build_dependencies, python_env.dependencies]):
-        # Create a temporary requirements file in the model directory to resolve the references
-        # in it correctly.
-        tmp_req_file = f"requirements.{uuid.uuid4().hex}.txt"
-        local_model_path.joinpath(tmp_req_file).write_text("\n".join(deps))
-        try:
+        with TempDir() as t:
+            # Create a temporary requirements file in the model directory to resolve the references
+            # in it correctly. To do this, we must first symlink or copy the model directory's
+            # contents to a temporary location for compatibility with deployment tools that store
+            # models in a read-only mount
+            tmp_model_dir = t.path("model")
+            os.makedirs(tmp_model_dir)
+            try:
+                for model_item in os.listdir(local_model_path):
+                    os.symlink(
+                        src=os.path.join(local_model_path, model_item),
+                        dst=os.path.join(tmp_model_dir, model_item),
+                    )
+            except Exception as e:
+                _logger.warning(
+                    "Failed to symlink model directory during dependency installation"
+                    " Copying instead. Exception: %s",
+                    e,
+                )
+                shutil.rmtree(tmp_model_dir)
+                _copy_model_to_writeable_destination(local_model_path, tmp_model_dir)
+
+            tmp_req_file = f"requirements.{uuid.uuid4().hex}.txt"
+            Path(tmp_model_dir).joinpath(tmp_req_file).write_text("\n".join(deps))
             cmd = _join_commands(activate_cmd, f"python -m pip install --quiet -r {tmp_req_file}")
-            _exec_cmd(cmd, capture_output=capture_output, cwd=local_model_path, extra_env=extra_env)
-        finally:
-            local_model_path.joinpath(tmp_req_file).unlink()
+            _exec_cmd(cmd, capture_output=capture_output, cwd=tmp_model_dir, extra_env=extra_env)
 
     return activate_cmd
 
 
+def _copy_model_to_writeable_destination(model_src, dst):
+    """
+    Copies the specified `model_src` directory, which may be read-only, to the writeable `dst`
+    directory.
+    """
+    os.makedirs(dst, exist_ok=True)
+    for model_item in os.listdir(model_src):
+        # Copy individual files and subdirectories, rather than using `shutil.copytree()`
+        # because `shutil.copytree()` will apply the permissions from the source directory,
+        # which may be read-only
+        if os.path.isdir(model_item):
+            copy_fn = shutil.copytree
+        else:
+            copy_fn = shutil.copy2
+
+        copy_fn(
+            src=os.path.join(model_src, model_item),
+            dst=os.path.join(dst, model_item),
+        )
+
+
 def _get_virtualenv_extra_env_vars(env_root_dir=None):
     extra_env = {
         # PIP_NO_INPUT=1 makes pip run in non-interactive mode,
